/*
 File				Event.cpp

 Author:			Lukasz Beca
 Created:			--/--/--

 Modified:		07/22/98
 By:				Pawel Roman
 Changes:			Clean-up, DLL and SO version

 Modified:		04/15/98
 By:				Grzegorz Lewandowski
 Changes:			new version of registerApp
					registerWinApp added

 Modified:		04/13/98
 By:				Grzegorz Lewandowski
 Changes:			Win32 port
					writeAll added
					sendEvent returns HRESULT value
*/

#include "Event.h"

#if !defined( _WIN32 )
#include <sys/time.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/ioctl.h>
#include <netinet/in.h>
#include <netdb.h>
#include <stdlib.h>
#include <unistd.h>

#define SOCKET_ERROR -1
#define ERROR_READ_FAULT -1
#define ERROR_WRITE_FAULT -1
#define ERROR_INVALID_PARAMETER -1
#define INVALID_SOCKET -1
#define ERROR_HANDLE_EOF 0
#define S_OK 1

#define closesocket close
#endif

/*
 readAll
*/
HRESULT readAll( SOCKET fd, char* pBuffer, size_t Length )
{
	int nResult;
	do
	{
		nResult = recv( fd, pBuffer, Length, 0 );
		if( nResult > 0 )
		{
			Length -= nResult;
			pBuffer = pBuffer + nResult;
		}
		else
		{
			if( nResult == SOCKET_ERROR )
			{
				return ERROR_READ_FAULT;
			}
			return ERROR_HANDLE_EOF;			
		}
	}
	while( Length > 0 );
	return S_OK;
}


/*
 writeAll
*/
HRESULT writeAll( SOCKET fd, char* pBuffer, size_t Length )
{
	int nResult;
	do
	{
		nResult = send( fd, pBuffer, Length, 0 );
		if( nResult > 0 )
		{
			Length -= nResult;
			pBuffer = pBuffer + nResult;
		}
		else
		{
			if( nResult == SOCKET_ERROR )
			{
				return ERROR_WRITE_FAULT;
			}
			return ERROR_HANDLE_EOF;
		}
	}
	while( Length > 0 );
	return S_OK;
}


/*
 registerApp
*/
SOCKET registerApp( int argc, char *argv[] )
{
	int AID;
	int SID;
	int AT;
	int port;
	
	SOCKET sockfd;
	struct sockaddr_in serv_addr;
	char *pBuffer;
	long lToSend;
	int nResult;
	char HostName[ 127 ];
	struct hostent *pHostInfo;

	/* retrieve command line parameters */
	if( argc < 5 )
	{
		return INVALID_SOCKET;
	}
	
	if( argv[ 1 ] != NULL )
	{
		AID = atoi( argv[ 1 ] );
	}
	else
	{
		return INVALID_SOCKET;
	}
	
	if( argv[ 2 ] != NULL )
	{
		SID = atoi( argv[ 2 ] );
	}
	else
	{
		return INVALID_SOCKET;
	}

	if( argv[ 3 ] != NULL )
	{
		AT = atoi( argv[ 3 ] );
	}
	else
	{
		return INVALID_SOCKET;
	}

	if( argv[ 4 ] != NULL )
	{
		port = atoi( argv[ 4 ] );
	}
	else
	{
		return INVALID_SOCKET;
	}

	/* obtain address of the local host */
	if( gethostname( HostName, 127 ) == SOCKET_ERROR )
	{
		return INVALID_SOCKET;
	}
	
	if( ( pHostInfo = gethostbyname( HostName ) ) == NULL )
	{
		return INVALID_SOCKET;
	}

	/* create socket */
	serv_addr.sin_family = AF_INET;
	serv_addr.sin_addr = *(struct in_addr*)( pHostInfo->h_addr ); 
	serv_addr.sin_port = htons( (unsigned short)port );

	if( ( sockfd = socket( AF_INET, SOCK_STREAM, 0 ) ) == INVALID_SOCKET )
	{
		return INVALID_SOCKET;
	}

	/* connect to TANGO */
	if( connect( sockfd, ( struct sockaddr* )&serv_addr, sizeof( serv_addr ) ) == SOCKET_ERROR )
	{
		closesocket( sockfd );
		return INVALID_SOCKET;
	}

	/* send identification */
	/*   AID */
	lToSend = AID;
	lToSend = htonl( lToSend );
	pBuffer = (char*)&lToSend;
	if( ( nResult = send( sockfd, pBuffer, sizeof( lToSend ), 0 ) ) == SOCKET_ERROR )
	{
		closesocket( sockfd );
		return INVALID_SOCKET;
	}

	/*  SID */
	lToSend = SID;
	lToSend = htonl( lToSend );
	pBuffer = (char*)&lToSend;
	if( ( nResult = send( sockfd, pBuffer, sizeof( lToSend ), 0 ) ) == SOCKET_ERROR )
	{
		closesocket( sockfd );
		return INVALID_SOCKET;
	}

	/*  AT */
	lToSend = AT;
	lToSend = htonl( lToSend );
	pBuffer = (char*)&lToSend;
	if( ( nResult = send( sockfd, pBuffer, sizeof( lToSend ), 0 ) ) == SOCKET_ERROR )
	{
		closesocket( sockfd );
		return INVALID_SOCKET;
	}
	
	return sockfd;
}


/*
 registerWinApp
*/
#if defined( _WIN32 )
SOCKET registerWinApp( LPSTR lpCmdLine )
{
	char szTokens[] = " ";
	char* szToken;

	int AID = -1;
	int SID = -1;
	int AT = -1;
	int port = -1;

	SOCKET sockfd;
	struct sockaddr_in serv_addr;
	char *pBuffer;
	long lToSend;
	int nResult;
	char HostName[ 127 ];
	struct hostent *pHostInfo;

	/* retrieve command line parameters */
	szToken = strtok( lpCmdLine, szTokens ); 
	while( szToken != NULL )
	{
		if( AID == -1 )
		{
			AID = atoi( szToken );
		}
		else if( SID == -1 )
		{
			SID = atoi( szToken );
		}
		else if( AT == -1 )
		{
			AT = atoi( szToken );
		}
		else if( port == -1 )
		{
			port = atoi( szToken );
		}
		else
		{
			break;
		}
		szToken = strtok( NULL, szTokens ); 
	}

	if( ( AID == -1 ) || ( SID == -1 ) || ( AT == -1 ) || ( port == -1 ) )
	{
		return INVALID_SOCKET;
	}

	/* obtain address of the local host */
	if( gethostname( HostName, 127 ) == SOCKET_ERROR )
	{
		return INVALID_SOCKET;
	}


	if( ( pHostInfo = gethostbyname( HostName ) ) == NULL )
	{
		return INVALID_SOCKET;
	}

	/* create socket */
	serv_addr.sin_family = AF_INET;
	serv_addr.sin_addr = *(struct in_addr*)( pHostInfo->h_addr ); 
	serv_addr.sin_port = htons( (unsigned short)port );

	if( ( sockfd = socket( AF_INET, SOCK_STREAM, 0 ) ) == INVALID_SOCKET )
	{
		return INVALID_SOCKET;
	}

	/* connect to TANGO */
	if( connect( sockfd, ( struct sockaddr* )&serv_addr, sizeof( serv_addr ) ) == SOCKET_ERROR )
	{
		closesocket( sockfd );
		return INVALID_SOCKET;
	}

	/* send identification */
	/*   AID */
	lToSend = AID;
	lToSend = htonl( lToSend );
	pBuffer = (char*)&lToSend;
	if( ( nResult = send( sockfd, pBuffer, sizeof( lToSend ), 0 ) ) == SOCKET_ERROR )
	{
		closesocket( sockfd );
		return INVALID_SOCKET;
	}

	/*  SID */
	lToSend = SID;
	lToSend = htonl( lToSend );
	pBuffer = (char*)&lToSend;
	if( ( nResult = send( sockfd, pBuffer, sizeof( lToSend ), 0 ) ) == SOCKET_ERROR )
	{
		closesocket( sockfd );
		return INVALID_SOCKET;
	}

	/*  AT */
	lToSend = AT;
	lToSend = htonl( lToSend );
	pBuffer = (char*)&lToSend;
	if( ( nResult = send( sockfd, pBuffer, sizeof( lToSend ), 0 ) ) == SOCKET_ERROR )
	{
		closesocket( sockfd );
		return INVALID_SOCKET;
	}
	
	return sockfd;
}
#endif

/*
 sendEvent
*/
HRESULT sendEvent( SOCKET sockfd, struct Event *pev )
{
	int buffer;
	HRESULT hr;

	if( pev == NULL )
	{
		return ERROR_INVALID_PARAMETER;
	}

	if( pev->data == NULL )
	{
		return ERROR_INVALID_PARAMETER;
	}

	buffer = htonl( pev->type );

	hr = writeAll( sockfd, (char*)&buffer, sizeof( int ) );
	if( hr != S_OK )
	{
		return ERROR_WRITE_FAULT;
	}

	buffer = htonl( pev->len );
	hr = writeAll( sockfd, (char*)&buffer, sizeof( int ) );
	if( hr != S_OK )
	{
		return ERROR_WRITE_FAULT;
	}

	hr = writeAll( sockfd, pev->data, pev->len );
	if( hr != S_OK )
	{
		return ERROR_WRITE_FAULT;
	}

	return S_OK;
}


/*
 receiveEvent
*/
struct Event* receiveEvent( SOCKET sockfd )
{
	HRESULT hr;
	char Buffer[ 4 ];
	int *a;
	int type;
	int len;
	char *data;
	struct Event *r; 
	
	hr = readAll( sockfd, Buffer, 4 );
	if( hr != S_OK )
	{
		return NULL;
	}
	a = (int*)Buffer;
	type = ntohl( *a );
	
	hr = readAll( sockfd, Buffer, 4 );
	if( hr != S_OK )
	{
		return NULL;
	}
	a = (int*)Buffer;
	len = ntohl( *a );
	
	data = ( char* )malloc( len * sizeof( char ) );

	if( data == NULL )
	{
		return NULL;
	}
	
	hr = readAll( sockfd, data, len );
	if( hr != S_OK )
	{
		free( data );
		return NULL;
	}
	r = createEvent( type, len, data );
	if( r == NULL )
	{
		free( data );
		return NULL;
	}
	
	if( type == PING )
	{
		if( sendEvent( sockfd, r ) != S_OK )
		{
			free( data );
			return NULL;
		}
	}

	free( data );
	return r;
}


/*
 createEvent
*/
struct Event* createEvent( int type, int len, char *data )
{
	int i;
	struct Event *Result;
	
	if( data == NULL )
	{
		return NULL;
	}

	Result = NULL;
	Result = ( struct Event* )malloc( sizeof( struct Event ) );
	if( Result == NULL )
	{
		return NULL;
	}
	Result->type = type;
	Result->len = len;
	Result->data = NULL;
	Result->data = ( char* )malloc( len * sizeof( char ) );
	if( Result->data == NULL )
	{
		return NULL;
	}

	for( i = 0; i < len; i ++ )
	{
		Result->data[ i ] = data[ i ];
	}
	
	return Result;
}


/*
 destroyEvent
*/
void destroyEvent( struct Event *pev )
{
	if( pev != NULL )
	{
		if( pev->data != NULL )
		{
			free( pev->data );
		}
		free( pev );
		pev = NULL;
	}
}


/*
 getType
*/
int getType( struct Event *pev )
{
	if( pev != NULL )
	{
		return pev->type;
	}
	else
	{
		return ERROR_INVALID_PARAMETER;
	}
}


/*
 getLen
*/
int getLen( struct Event *pev )
{
	if( pev != NULL )
	{
		return pev->len;
	}
	else
	{
		return ERROR_INVALID_PARAMETER;
	}
}


/*
 getData
*/
char *getData( struct Event *pev )
{
	int nLength;
	int i;
	char *pBuffer;

	if( pev == NULL )
	{
		return NULL;
	}
	if( pev->data == NULL )
	{
		return NULL;
	}

	nLength = getLen( pev );
	pBuffer = ( char* )malloc( nLength * sizeof( char ) );

	if( pBuffer == NULL )
	{
		return NULL;
	}

	for( i = 0; i < nLength; i ++ ) 
	{
		pBuffer[ i ] = pev->data[ i ];
	}
	
	return pBuffer;
}
