/***************************************************************************
  FILE: QueueHandler.java, TomaszMajor-10Feb99
  PURPOSE: main interface
  $Id: QueueHandler.java,v 1.2 1999/04/22 21:47:03 toma Exp $
  Component: Plugin, Interfaces
***************************************************************************/


package webwisdom.tango;


import java.util.Vector;


/**Queue that sequentially precess its content.
   Processed items are removed.*/
public final class QueueHandler implements Runnable
{
    private static final String CL="QueueHandler";


    /**Queue identifier.*/
    private String name;

    /**Interface which is going to receive elements of the queue.*/
    private Handler handler;

    private Vector queue;

    /**Used to unblock*/
    private boolean isRunning;
    private boolean isStopped;


    public QueueHandler(String n,Handler h)
    {
      name=n;
      handler=h;
      queue=new Vector();
      isRunning=false;
      isStopped=false;
    }// QueueHandler _______________________________________________________

    
    public String getName()
    {
      return name;
    }// getName ____________________________________________________________


    public void add(Object o)
    {
      //System.out.println(name+".add("+o+")");
      synchronized(queue)
      {
	queue.addElement(o);
	queue.notifyAll();
      }
    }// add ________________________________________________________________


    public void run()
    {
      //System.out.println(name+": started");
      isRunning=true;
      while(true)
      {
	try
	{
	  Object o;
	  synchronized(queue)
	  {
	    while((queue.size()==0)&&(isRunning))
	      queue.wait();
	    if(!isRunning)
	      break;
	    o=queue.elementAt(0);
	    queue.removeElementAt(0);
	  }
	  //System.out.println(name+": processed: "+o);
	  handler.handle(o);
	}
	catch(InterruptedException e)
	{
	  System.err.println(name+": unexpected interruption!");
	  e.printStackTrace();
	}
      }
      isStopped=true;
      //System.out.println(name+": finished.");
    }// run ________________________________________________________________


    public boolean isRunning()
    {
      return isRunning;
    }// isRunning __________________________________________________________


    public boolean isStopped()
    {
      return isStopped;
    }// isStopped __________________________________________________________


    public void stop()
    {
      isRunning=false;
      synchronized(queue)
      {
	queue.notify();
      }
    }// stop _______________________________________________________________


    public String toString()
    {
      return CL+"["+name+"]";
    }// toString ___________________________________________________________


}// QueueHandler ///////////////////////////////////////////////////////////


