/***************************************************************************
  FILE: TAgent.java, PiotrSokolowski,TomaszMajor-10Apr98
  PURPOSE: main interface
  $Id: TAgent.java,v 1.30 1999/04/22 21:47:06 toma Exp $
  Component: Plugin, Interfaces
***************************************************************************/


package webwisdom.tango;


import java.applet.Applet;
import java.util.Vector;
import java.util.Hashtable;
import java.util.Enumeration;
import java.util.NoSuchElementException;
import java.io.ByteArrayInputStream;
import java.io.DataInputStream;
import java.io.IOException;

import webwisdom.tango.consts.CAConsts;
import webwisdom.tango.consts.Const;
import webwisdom.tango.messages.CAppAnswerEventMessage;
import webwisdom.tango.messages.CAppRequestEventMessage;
import webwisdom.tango.messages.AppEventMessage;
import webwisdom.tango.structures.AppletHandle;
import webwisdom.tango.extras.BvListener;

import netscape.javascript.JSObject;


/**Blocks quaries made by control messages until response is available.*/
final class SyncQuery
{
    private static final String CL="SyncQuery";


    private TAgent agent;

    /**request, same message always reused*/
    private CAppRequestEventMessage reqMsg;

    /**response, null if not yet available*/
    private CAppAnswerEventMessage resMsg;

    /**used to unblock*/
    private boolean quitting=false;


    /**
     *@param t type of a query message: one of CAConsts*/
    public SyncQuery(TAgent a,byte t)
    {
      //System.out.println(CL+".SyncQuery("+a+","+t+")");
      agent=a;

      byte b[]=new byte[1];
      b[0]=t;
      reqMsg=new CAppRequestEventMessage(b);
    }// SyncQuery __________________________________________________________
  

    public CAppAnswerEventMessage query()
    {
      resMsg=null;
      agent.send(reqMsg);
      synchronized(this)
      {
	try
	{
	  while((resMsg==null)&&(!quitting))
	    wait();
	}
	catch(InterruptedException e)
	{
	  System.err.println(CL+".query(): unexpected interruption! returning null");
	  //System.out.println(CL+".query(): interrupted");
	}
      }
      return resMsg;
    }// query ______________________________________________________________
    
    
    /**Called by TAgent when the appropriate type of response
       is received for this SyncQuery.*/
    public synchronized void response(CAppAnswerEventMessage m)
    {
      //System.out.println(CL+".response("+m+")");
      resMsg=m;
      notifyAll();
    }// response ___________________________________________________________


    /**Unblocks if waiting. query() will return null.*/
    public synchronized void quit()
    {
      quitting=true;
      notifyAll();
    }// quit _______________________________________________________________


    public String toString()
    {
      return CL+"["+reqMsg+"]";
    }// toString ___________________________________________________________


}// SyncQuery //////////////////////////////////////////////////////////////




/**Translates general Handler interface to TAgent.handleSysMsgAsync().*/
final class SysMsgHandler implements Handler
{


    private TAgent agent;


    public SysMsgHandler(TAgent a)
    {
      agent=a;
    }// SysMsgHandler ______________________________________________________


    public void handle(Object o)
    {
      agent.handleSysMsgAsync((CAppAnswerEventMessage)o);
    }// handle _____________________________________________________________


}// SysMsgHandler //////////////////////////////////////////////////////////




/**Translates general Handler interface to TAgent.handlePriMsg().*/
final class PriMsgHandler implements Handler
{


    private TAgent agent;


    public PriMsgHandler(TAgent a)
    {
      agent=a;
    }// PriMsgHandler ______________________________________________________


    public void handle(Object o)
    {
      agent.handlePriMsg((AppEventMessage)o);
    }// handle _____________________________________________________________


}// PriMsgHandler //////////////////////////////////////////////////////////




/**General Tango system proxy.
   This interface is the same for all application types.*/
public class TAgent implements TLListener
{
    private static final String CL="TAgent";


    /**Default channel id.*/
    public static final int NONCHANNEL_CHANNELID=-128;


    private TLAgent lAgent;
    private QueueHandler priMsgQueue,sysMsgQueue;


    protected Hashtable channels=new Hashtable();
    protected Vector contList=new Vector();
    protected Vector bvList=new Vector();


    protected Boolean isMaster;
    private Boolean a_isMaster;
    protected String masterName;
    protected String userName;
    private Vector participants;
    private Vector a_participants;
    protected Hashtable partipuids;


    private SyncQuery sq_isMaster;
    private SyncQuery sq_masterName;
    private SyncQuery sq_userName;
    private SyncQuery sq_participants;
    private SyncQuery sq_partipuid;

    
    /**Tango version is specified in the following format:
       <blockquote><pre>
       &#060<i>major version</i>&#062<b>.</b>&#060<i>minor version</i>&#062<b>.</b>&#060<i>maintenance release</i>&#062<b>.</b>&#060<i>bug fixes</i>&#062
       </pre></blockquote>
       only <major version>.<minor version> may count for compatibility.
     *@return Tango version*/
    public static String getVersion()
    {
      return webwisdom.tango.structures.LocalBase.getPluginVersion();
    }// getVersion _________________________________________________________


    /**Creates uninitialized TAgent.
     *@see setTLAgent*/
    public TAgent()
    {
      //System.out.println(CL+".TAgent()");

      sq_isMaster=new SyncQuery(this,CAConsts.ISMASTER);
      sq_masterName=new SyncQuery(this,CAConsts.MASTERNAME);
      sq_userName=new SyncQuery(this,CAConsts.USERNAME);
      sq_participants=new SyncQuery(this,CAConsts.PARTICIPANTS);
      sq_partipuid=new SyncQuery(this,CAConsts.PARTIPUID);
    }// TAgent _____________________________________________________________


    /**Initializes TAgent created with nonparam constructor.
     *@param a low level Tango system proxy
     specific for the application type*/
    public void setTLAgent(TLAgent a)
    {
      //System.out.println(CL+".setTLAgent("+a+")");


      String name=CL+"["+a.getSessionId()+"].sysMsgQueue";
      sysMsgQueue=new QueueHandler(name,new SysMsgHandler(this));
      (new Thread(sysMsgQueue,name)).start();

      name=CL+"["+a.getSessionId()+"].priMsgQueue";
      priMsgQueue=new QueueHandler(name,new PriMsgHandler(this));
      Thread pt=new Thread(priMsgQueue,name);
      pt.setPriority(Math.max(pt.getPriority()-1,Thread.MIN_PRIORITY));
      pt.start();


      lAgent=a;
      lAgent.setTLListener(this);

      //System.out.println(CL+".setTLAgent().");
    }// void setTLAgent ____________________________________________________


    /**Creates TAgent.
     *@param a low level Tango system proxy
     specific for the application type
     *@see setTLAgent*/
    public TAgent(TLAgent a)
    {
      this();
      setTLAgent(a);
    }// TAgent _____________________________________________________________


    /**Leaves Tango system. Cannot join Tango again.*/
    public void exit()
    {
      //System.out.println(CL+".exit()");

      lAgent.exit();

      sq_isMaster.quit();
      sq_masterName.quit();
      sq_userName.quit();
      sq_participants.quit();
      sq_partipuid.quit();

      priMsgQueue.stop();
      sysMsgQueue.stop();

      //System.out.println(CL+".exit().");
    }// exit _______________________________________________________________


    /**Adds a recipient of the session data from the given channels.
     *@see #send(int,)*/
    public void addTDataListener(int chId,TDataListener l)
    {
      //System.out.println(CL+".addTDataListener("+chId+","+l+")");
      Integer k=new Integer(chId);
      Vector dataList=(Vector)channels.get(k);
      if(dataList==null)
      {
	dataList=new Vector();
	channels.put(k,dataList);
      }
      dataList.removeElement(l);
      dataList.addElement(l);
    }// addTDataListener ___________________________________________________


    /**Adds a recipient of the session data.
     *@see #send()*/
    public void addTDataListener(TDataListener l)
    {
      addTDataListener(NONCHANNEL_CHANNELID,l);
    }// addTDataListener ___________________________________________________


    /**Removes given listener from the listener list.
     *@see #addTDataListener*/
    public void removeTDataListener(TDataListener l)
    {
      Enumeration e=channels.elements();
      while(e.hasMoreElements())
      {
	Vector dataList=(Vector)(e.nextElement());
	dataList.removeElement(l);
      }
    }// removeTDataListener ________________________________________________


    /**Adds a recipient of the Tango system notifications.*/
    public void addTControlListener(TControlListener l)
    {
      //System.out.println(CL+".addTControlListener("+l+")");
      contList.removeElement(l);
      contList.addElement(l);
    }// addTControlListener ________________________________________________


    /**Removes a recipient of the Tango system notifications.
     *@see #addTControlListener*/
    public void removeTControlListener(TControlListener l)
    {
      contList.removeElement(l);
    }// removeTControlListener _____________________________________________


    /**Adds a recipient of BV notifications.*/
    public void addBvListener(BvListener l)
    {
      //System.out.println(CL+".addBvListener("+l+")");
      bvList.removeElement(l);
      bvList.addElement(l);
    }// addBvListener ______________________________________________________


    /**Removes a recipient of BV notifications.
     *@see #addBvListener*/
    public void removeBvListener(BvListener l)
    {
      bvList.removeElement(l);
    }// removeBvListener ___________________________________________________


    /**Interface for low level mechanisms.
       <p>Note: This is not a public interface.
     *@see TLAgent*/
    public final void receive(AppEventMessage m)
    {
      //System.out.println(CL+".receive("+m+")");
      int type=m.getType();
      if(type==Const.DATA)
      {
	priMsgQueue.add(m);
      }
      else if(type==Const.CONTROL)
      {
	//System.out.println(CL+".receive("+m+"): CONTROL");
	CAppAnswerEventMessage mm=null;
	try
	{
	  mm=(CAppAnswerEventMessage)m;
	  handleSysMsgSync(mm);
	  sysMsgQueue.add(mm);
	}
	catch(ClassCastException e)
	{
	  System.err.println(CL+".receive("+m+"): "+e);
	}
      }
      else
      {
	System.err.println(CL+".receive("+m+"): no such type: "+type);
      }
    }// receive ____________________________________________________________


    final void handlePriMsg(AppEventMessage m)
    {
      //System.out.println(CL+".handlePriMsg("+m+")");
      byte[] data=m.getData();
      int chId=(int)data[0];
      int chDataLen=data.length-1;
      byte[] chData=new byte[chDataLen];
      System.arraycopy(data,1,chData,0,chDataLen);
      Vector dataList=(Vector)(channels.get(new Integer(chId)));
      if(dataList!=null)
	for(int i=0;i<dataList.size();i++)
	  ((TDataListener)dataList.elementAt(i)).receive(chData);
    }// handlePriMsg _______________________________________________________


    private final synchronized void handleSysMsgSync(CAppAnswerEventMessage m)
    {
      //System.out.println(CL+".handleSysMsgSync("+m+")");
      switch(m.getAnswerType())
      {
	case CAConsts.PARTICIPANTS:
	{
	  //System.out.println(CL+".handleSysMsgSync("+m+"): PARTICIPANTS");
	  partipuids=null;
	  a_participants=participants;
	  participants=m.getParticipants();
	  //System.out.println("\tparticipants="+vectorToString(participants));
	  sq_participants.response(m);
	  break;
	}
	case CAConsts.PARTIPUID:
	{
	  //System.out.println(CL+".handleSysMsgSync("+m+"): PARTIPUID");
	  byte b[]=m.getData();
	  ByteArrayInputStream bais=new ByteArrayInputStream(b);
	  DataInputStream dis=new DataInputStream(bais);
	  partipuids=new Hashtable();//.clear();
	  try
	  {
	    while(dis.available()>0)
	    {
	      Integer i=new Integer(dis.readInt());
	      String s=dis.readUTF();
	      //System.out.println("\t("+i+","+s+")");
	      partipuids.put(s,i);
	    }
	  }
	  catch(IOException e)
	  {
	    System.err.println(CL+".handleSysMsgSync("+m+")"+
			       ": error processing PARTIPUID message!: "+e);
	    e.printStackTrace();
	  }
	  sq_partipuid.response(m);
	  break;
	}
	case CAConsts.ISMASTER:
	{
	  //System.out.println(CL+".handleSysMsgSync("+m+"): ISMASTER");
	  masterName=null;
	  a_isMaster=isMaster;
	  isMaster=new Boolean(m.isMaster());
	  //System.out.println("\tisMaster="+isMaster);
	  sq_isMaster.response(m);
	  break;
	}
	case CAConsts.MASTERNAME:
	{
	  //System.out.println(CL+".handleSysMsgSync("+m+"): MASTERNAME");
	  masterName=m.getMastername();
	  //System.out.println("\tmasterName="+masterName);
	  sq_masterName.response(m);
	  break;
	}
	case CAConsts.USERNAME:
	{
	  //System.out.println(CL+".handleSysMsgSync("+m+"): USERNAME");
	  userName=m.getUsername();
	  //System.out.println("\tuserName="+userName);
	  sq_userName.response(m);
	  break;
	}
	default:
	{
	  ;//System.err.println(CL+".handleSysMsgSync("+m+"): unknown message!");
	}
      }
    }// handleSysMsgSync ___________________________________________________


    /**This function assumes that handleSyncControl()
       has been called with the same argument beforehand.*/
    final void handleSysMsgAsync(CAppAnswerEventMessage m)
    {
      //System.out.println(CL+".handleSysMsgAsync("+m+")");
      switch(m.getAnswerType())
      {
	case CAConsts.PARTICIPANTS:
	{
	  //System.out.println(CL+".handleSysMsgAsync("+m+"): PARTICIPANTS");
	  //System.out.println("\told: "+vectorToString(a_participants));
	  //System.out.println("\tnew: "+vectorToString(participants));
	  if(a_participants==null)
	  {
	    a_participants=participants;
	    break;
	  }
	  for(int i=0;i<a_participants.size();i++)
	  {
	    Object p=a_participants.elementAt(i);
	    if(!participants.contains(p))
	      for(int li=0;li<contList.size();li++)
		((TControlListener)contList.elementAt(li)).participantLeft((String)p);
	  }
	  for(int i=0;i<participants.size();i++)
	  {
	    Object p=participants.elementAt(i);
	    if(!a_participants.contains(p))
	      for(int li=0;li<contList.size();li++)
		((TControlListener)contList.elementAt(li)).participantJoined((String)p);
	  }
	  a_participants=participants;
	  break;
	}
	case CAConsts.ISMASTER:
	{
	  //System.out.println(CL+".handleSysMsgAsync("+m+"): ISMASTER");
	  //System.out.println("\told: "+a_isMaster);
	  //System.out.println("\tnew: "+isMaster);
	  if(a_isMaster==null)
	  {
	    a_isMaster=isMaster;
	    break;
	  }
	  if(!a_isMaster.equals(isMaster))
	  {
	    for(int i=0;i<contList.size();i++)
	      ((TControlListener)contList.elementAt(i)).masterChanged(isMaster(),getMasterName());
	    a_isMaster=isMaster;
	  }
	  break;
	}
	case CAConsts.FOCUS:
	{
	  //System.out.println(CL+".handleSysMsgAsync("+m+"): FOCUS");
	  break;
	}
	case CAConsts.NEW:
	{
	  //System.out.println(CL+".handleSysMsgAsync("+m+"): NEW");
	  byte b[]=m.getData();
	  ByteArrayInputStream bais=new ByteArrayInputStream(b);
	  DataInputStream dis=new DataInputStream(bais);
	  try
	  {
	    int mH=dis.readInt();
	    int mI=dis.readInt();
	    String mN=getStringForBv(dis);
	    //System.out.println("\t("+mN+","+mI+","+mH+")");
	    for(int i=0;i<bvList.size();i++)
	      ((BvListener)bvList.elementAt(i)).participantJoined(mN,mI,mH);
	  }
	  catch(IOException e)
	  {
	    System.err.println(CL+".handleSysMsgSync("+m+")"+
			       ": error processing NEW message!: "+e);
	    e.printStackTrace();
	  }
	  break;
	}
	case CAConsts.DELETE:
	{
	  //System.out.println(CL+".handleSysMsgAsync("+m+"): DELETE");
	  byte b[]=m.getData();
	  ByteArrayInputStream bais=new ByteArrayInputStream(b);
	  DataInputStream dis=new DataInputStream(bais);
	  try
	  {
	    int mH=dis.readInt();
	    int mI=dis.readInt();
	    String mN=getStringForBv(dis);
	    //System.out.println("\t("+mN+","+mI+","+mH+")");
	    for(int i=0;i<bvList.size();i++)
	      ((BvListener)bvList.elementAt(i)).participantLeft(mN,mI,mH);
	  }
	  catch(IOException e)
	  {
	    System.err.println(CL+".handleSysMsgSync("+m+")"+
			       ": error processing DELETE message!: "+e);
	    e.printStackTrace();
	  }
	  break;
	}
	case CAConsts.JOIN:
	{
	  //System.out.println(CL+".handleSysMsgAsync("+m+"): JOIN");
	  byte b[]=m.getData();
	  ByteArrayInputStream bais=new ByteArrayInputStream(b);
	  DataInputStream dis=new DataInputStream(bais);
	  try
	  {
	    byte nP=dis.readByte();
	    int mH=dis.readInt();
	    int mI=dis.readInt();
	    String mN=getStringForBv(dis);
	    System.out.println("\t["+nP+"]("+mN+","+mI+","+mH+")");
	    for(int i=0;i<bvList.size();i++)
	      ((BvListener)bvList.elementAt(i)).connected(mN,mI,mH);
	  }
	  catch(IOException e)
	  {
	    System.err.println(CL+".handleSysMsgSync("+m+")"+
			       ": error processing JOIN message!: "+e);
	    e.printStackTrace();
	  }
	  break;
	}
	case CAConsts.LEAVE:
	{
	  //System.out.println(CL+".handleSysMsgAsync("+m+"): LEAVE");
	  for(int i=0;i<bvList.size();i++)
	    ((BvListener)bvList.elementAt(i)).disconnected();
	  break;
	}
	case CAConsts.TERMINATE:
	{
	  //System.out.println(CL+".handleSysMsgAsync("+m+"): TERMINATE");
	  break;
	}
	default:
	{
	  ;//System.err.println(CL+".handleSysMsgAsync("+m+"): unknown message!");
	}
      }
    }// handleSysMsgAsync __________________________________________________


    private String getStringForBv(DataInputStream s) throws IOException
    {
      int len=s.readByte();
      byte[] bytes=new byte[len];
      s.readFully(bytes);
      String string=new String(bytes);
      return string;
    }// getStringForBv _____________________________________________________


    /**Interface for SyncQuery*/
    void send(AppEventMessage m)
    {
      //System.err.println(CL+".send("+m+")");
      lAgent.send(m);
    }// send _______________________________________________________________


    /**Sends data to the other participants of the same session.
       This interface facilitate creation of compound applications
       containing multiple subapplications.
       Do not use multiple channels for a single application.
     *@param chId channel to send data into
     *@param chData data to be sent
     *@see #addTDataListener*/
    public void send(int chId,byte[] chData)
    {
      //System.out.println(CL+".send("+chId+",byte["+chData.length+"])");
      int chDataLen=chData.length;
      byte[] data=new byte[1+chDataLen];
      data[0]=(byte)chId;
      System.arraycopy(chData,0,data,1,chDataLen);

      lAgent.send(new AppEventMessage(Const.DATA,data));
    }// send _______________________________________________________________


    /**Sends data to the specified participants.
       Do not use multiple channels for a single application.
     *@param chId channel to send data into
     *@param part recipient names
     *@param chData data to be sent
     *@see #send(int,byte[])*/
    public void selectiveSend(int chId,String[] part,byte[] chData)
    {
      //System.out.println(CL+".selectiveSend("+chId+",String["+part.length+"],byte["+chData.length+"])");
      int n=part.length;
      int[] uids=new int[n];
      for(int i=0;i<n;i++)
	uids[i]=getUID(part[i]);

      int chDataLen=chData.length;
      byte[] data=new byte[1+chDataLen];
      data[0]=(byte)chId;
      System.arraycopy(chData,0,data,1,chDataLen);

      lAgent.send(uids,new AppEventMessage(Const.DATA,data));
    }// selectiveSend ______________________________________________________


    /**Sends data to the other participants of the same session.
     *@param data data to be sent
     *@see #addTDataListener*/
    public void send(byte[] data)
    {
      send(NONCHANNEL_CHANNELID,data);
    }// send _______________________________________________________________


    /**Sends data to the specified participants.
     *@param part recipient names
     *@param data data to be sent
     *@see #send(byte[])*/
    public void selectiveSend(String[] part,byte[] data)
    {
      selectiveSend(NONCHANNEL_CHANNELID,part,data);
    }// selectiveSend ______________________________________________________


    /**Finds participant id.
     *@param n participant name
     *@return participant id
     *@exception NoSuchElementException when no participant with such name*/
    protected int getUID(String n) throws NoSuchElementException
    {
      //System.out.println(CL+".getUID("+n+")");
      if(partipuids==null)
	sq_partipuid.query();

      Integer uid=(Integer)partipuids.get(n);
      if(uid==null)
	throw new NoSuchElementException(n);
      //System.out.println(CL+".getUID("+n+")="+uid);
      return uid.intValue();
    }// getUID _____________________________________________________________


    /**Retrieves this user name.
     *@return user name*/
    public String getUserName()
    {
      //System.out.println(CL+".getUserName()");
      if(userName==null)
	sq_userName.query();
      //System.out.println(CL+".getUserName()="+userName);
      return userName;
    }// getUserName ________________________________________________________


    protected Vector getParticipants()
    {
      //System.out.println(CL+".getParticipants()");
      if(participants==null)
	sq_participants.query();
      //System.out.println(CL+".getParticipants()="+vectorToString(participants));
      return participants;
    }// getParticipants ____________________________________________________


    /**Retrieves all participants of this session.
     *@return participant names*/
    public String[] getParticipantNames()
    {
      //System.out.println(CL+".getParticipantNames()");
      return vectorToStringArray(getParticipants());
    }// getParticipantNames ________________________________________________


    /**Retrieves the master of this session.
     *@return the master name*/
    public String getMasterName()
    {
      //System.out.println(CL+".getMasterName()");
      if(masterName==null)
	sq_masterName.query();
      //System.out.println(CL+".getMasterName()="+masterName);
      return masterName;
    }// getMasterName ______________________________________________________


    /**Checks whether this participant is the master of this session.
     *@return master status*/
    public boolean isMaster()
    {
      //System.out.println(CL+".isMaster()");
      if(isMaster==null)
	sq_isMaster.query();
      //System.out.println(CL+".isMaster()="+isMaster);
      return isMaster.booleanValue();
    }// isMaster ___________________________________________________________


    /**Checks whether this application is allowed to use audio at this moment.
     *@return true if audio can be used*/
    public boolean isAudioAvailable()
    {
      //System.out.println(CL+".isAudioAvailable()");
      return webwisdom.tango.structures.LocalBase.isAudioAvailable();
    }// isAudioAvailable ___________________________________________________


    private static String[] vectorToStringArray(Vector v)
    {
      if(v==null)
	return null;
      int n=v.size();
      String[] a=new String[n];
      for(int i=0;i<n;i++)
	a[i]=v.elementAt(i).toString();
      return a;
    }// vectorToStringArray ________________________________________________


    private static String arrayToString(Object[] ss,
					String beg,String sep,String end)
    {
      int n;
      if(ss!=null)
	n=ss.length;
      else
	n=6;
      StringBuffer s=new StringBuffer(n*10);


      if(beg!=null)
	s.append(beg);

      if(ss!=null)
      {
	for(int i=0;i<n;i++)
	{
	  s.append(ss[i].toString());
	  if(i<(n-1))
	    s.append(sep);
	}
      }

      if(end!=null)
	s.append(end);


      return s.toString();
    }// arrayToString ______________________________________________________


    private static String arrayToString(Object[] s)
    {
      return arrayToString(s,"{",",","}");
    }// arrayToString ______________________________________________________


    private static String vectorToString(Vector v)
    {
      return arrayToString(vectorToStringArray(v));
    }// vectorToStringArray ________________________________________________


    public String toString()
    {
      return CL+"[sid="+lAgent.getSessionId()+
	",userName="+getUserName()+",isMaster="+isMaster()+"]";
    }// toString ___________________________________________________________


    public static void main(String[] args)
    {
      System.out.println(TAgent.getVersion());
    }// toString ___________________________________________________________


}// TAgent /////////////////////////////////////////////////////////////////


