/***************************************************************************
  FILE: TLAgentApplication.java, TomaszMajor-10Dec98
  PURPOSE: interface component for applications
  $Id: TLAgentApplication.java,v 1.5 1999/04/22 21:47:11 toma Exp $
  Component: Plugin, Interfaces
***************************************************************************/


package webwisdom.tango;


import java.net.*;
import java.io.*;

import webwisdom.tango.consts.Const;
import webwisdom.tango.consts.CAConsts;
import webwisdom.tango.messages.AppEventMessage;
import webwisdom.tango.messages.CAppAnswerEventMessage;


class TLAgentApplicationReceiver extends Thread
{
    private static final String CL="TLAgentApplicationReceiver";


    private String name;
    private DataInputStream inStream;
    private TLAgentApplication agent;


    public TLAgentApplicationReceiver(String n,
				      DataInputStream is,TLAgentApplication a)
    {
      super(n);
      name=n;
      inStream=is;
      agent=a;
    }// TLAgentApplicationReceiver _________________________________________


    public void run()
    {
      try
      {
	do
	{
	  System.out.println(name+": waiting");
	  int type=inStream.readInt();
	  int packetLen=inStream.readInt();
	  byte[] packetData=new byte[packetLen];
	  inStream.readFully(packetData);
	  AppEventMessage m=new AppEventMessage(type,packetData);
	  System.out.println(name+": processing byte["+packetData.length+"]");
	  //System.out.println(CL+": <"+new String(packetData,0));
	  agent.receive(m);
	}
	while(true);
      }
      catch(EOFException e)
      {
	//System.err.println(CL+": Server disconnected!");
	agent.exit();
      }
      catch(IOException e)
      {
	System.err.println(name+": "+e);
	e.printStackTrace();
      }
    }// run ________________________________________________________________


    public String toString()
    {
	return CL+"["+name+"]";
    }// toString ___________________________________________________________


}// TLAgentApplicationReceiver /////////////////////////////////////////////




/**Tango system proxy for applications.
   This class may be used to create TAgent.
   *@see TAgent*/
public final class TLAgentApplication implements TLAgent
{
    private static final String CL="TLAgentApplication";


    private Socket socket;
    private DataOutputStream outStream;
    private DataInputStream inStream;

    private TLAgentApplicationReceiver receiverThread;

    private int sessionId=-1;
    private TLListener lowlList;


    /**Registers an application in the Tango system.
     *@param args original arguments passed to the program
     *@exception TangoException if cannot connect to Tango*/
    public TLAgentApplication(String[] args)
      throws TangoException
    {
      try
      {
	register(args);
      }
      catch(IOException e)
      {
	throw new TangoException("cannot register: "+e.getMessage());
      }
    }// TLAgentApplication _________________________________________________


    public TLAgentApplication(int AID,int SID,int AT,int port)
      throws TangoException
    {
      try
      {
	register(AID,SID,AT,port);
      }
      catch(IOException e)
      {
	throw new TangoException("cannot register: "+e.getMessage());
      }
    }// TLAgentApplication _________________________________________________


    private void register(int AID,int SID,int AT,int port)
      throws TangoException,IOException
    {
      System.out.println(CL+".register("+AID+","+SID+","+AT+","+port+")");
      sessionId=SID;
      try
      {
	InetAddress host=InetAddress.getLocalHost();
	socket=new Socket(host,port); 
	outStream=new DataOutputStream(socket.getOutputStream());
	inStream=new DataInputStream(socket.getInputStream());
	/*System.out.println(CL+": connected to: "+socketToString(socket)+
	  " (localPort="+socket.getLocalPort()+")");*/
      }
      catch(IOException e)
      {
	throw new TangoException("port "+port+" unreachable");
      }


      String name=CL+"["+socket.getLocalPort()+"].rcvThr";
      receiverThread=new TLAgentApplicationReceiver(name,inStream,this);
      receiverThread.start();
      (Thread.currentThread()).setPriority(Thread.MIN_PRIORITY);
      //pt.setPriority(Math.max(pt.getPriority()-1,Thread.MIN_PRIORITY));


      //System.out.println(CL+".register1("+AID+","+SID+","+AT+","+port+")");
      outStream.writeInt(AID);
      outStream.writeInt(SID);
      outStream.writeInt(AT);
      outStream.flush();
      System.out.println(CL+".register().");
    }// register ___________________________________________________________


    private void register(String[] args)
      throws TangoException,IOException
    {
      if((args==null)||(args.length<5))
	throw new TangoException("missing arguments: "+args);


      int AID=0,SID,AT;
      int port;

      try
      {
	AID=Integer.valueOf(args[1]).intValue();
	SID=Integer.valueOf(args[2]).intValue();
	AT=Integer.valueOf(args[3]).intValue();
	port=Integer.valueOf(args[4]).intValue();

// 	for(int i=0;i<args.length;)
// 	{
// 	  String pn=args[i++],p=args[i++];
// 	  if(pn.equals("-AID"))
// 	  {
// 	    AID=Integer.valueOf(p).intValue();
// 	  }
// 	  else if(pn.equals("-SID"))
// 	  {
// 	    SID=Integer.valueOf(p).intValue();
// 	  }
// 	  else if(pn.equals("-AT"))
// 	  {
// 	    AT=Integer.valueOf(p).intValue();
// 	  }
// 	  else if(pn.equals("-port"))
// 	  {
// 	    port=Integer.valueOf(p).intValue();
// 	  }
// 	}
      }
      catch(NumberFormatException e)
      {
	throw new TangoException("wrong arguments: "+e.getMessage());
      }
      register(AID,SID,AT,port);
    }// register ___________________________________________________________


    /*interface for TLAgentApplicationReceiver*/
    void receive(AppEventMessage m)
    {     
      //System.out.println(CL+".receive("+m+")");
      if(m.getType()==Const.DATA)
      {
	System.out.println(CL+".receive("+m+"): DATA");
	if(lowlList!=null)
	  lowlList.receive(m);	
      }
      else if(m.getType()==Const.CONTROL)
      {
	System.out.println(CL+".receive("+m+"): CONTROL");
	AppEventMessage mm;
	byte[] b=m.getData();
	byte t=b[0];
	System.out.println(CL+".receive("+m+"): "+t+",byte["+b.length+"]");
	if((t==CAConsts.PARTICIPANTS)||
	   (t==CAConsts.PARTIPUID)||
	   (t==CAConsts.FOCUS)||
	   (t==CAConsts.ISMASTER)||
	   (t==CAConsts.MASTERNAME)||
	   (t==CAConsts.USERNAME))
	  mm=new CAppAnswerEventMessage(b);
	else
	  mm=m;
	System.out.println(CL+".receive("+m+"): mm="+mm);
	if(lowlList!=null)
	  lowlList.receive(mm);
      }
      else if(m.getType()==Const.PING)
      {
	System.out.println(CL+".receive("+m+"): PING");
	send(m);
      }
      else
      {
	System.err.println(CL+".receive("+m+"): unknown message type!");
      }
    }// receive ____________________________________________________________


    public void send(AppEventMessage m)
    {
      System.out.println(CL+".send("+m+")");
      try
      {
	m.send(outStream);
      }
      catch(IOException e)
      {
	System.err.println(CL+".send("+m+"): "+e);
	e.printStackTrace();
      }
    }// send _______________________________________________________________


    /**This function is not implemented*/
    public void send(int userlist[],AppEventMessage m)
    {
      //System.out.println(CL+".send("+arrayToString(userlist)+","+m+")");
      System.err.println(CL+".send(int[],AppEventMessage): redirecting to send(AppEventMessage)");
      send(m);
    }// send _______________________________________________________________


    public void setTLListener(TLListener tl)
    {
      //System.out.println(CL+".setTLListener("+tl+")");
      lowlList=tl;
    }// setTLListener ______________________________________________________


    public int getSessionId()
    {
      return sessionId;
    }// getSessionId _______________________________________________________


    public void exit()
    {
      System.out.println(CL+".exit()");

//       try
//       {
// 	if(socket!=null)
// 	  socket.close();
//       }
//       catch(IOException e)
//       {
//       }

//       if((receiverThread!=null)&&(receiverThread.isAlive()))
// 	receiverThread.stop();

      System.exit(0);
    }// exit _______________________________________________________________


    public String toString()
    {
      String host=socket.getInetAddress().getHostName();
      int port=socket.getPort();
      return CL+"[sid="+getSessionId()+","+host+":"+port+"]";
    }// toString ___________________________________________________________


}// TLAgentApplication /////////////////////////////////////////////////////


