// $Id: TangoBean.java,v 1.3 1998/09/22 18:34:27 beca Exp $
/*
File: TangoBean.java
Component: Plugin
Date            Author        Changes
Aug 05 98       Lukasz Beca   Created
*/

package webwisdom.tango.beans;

import java.awt.*;
import java.beans.*;
import java.util.*;
import java.io.*;
import webwisdom.tango.*;
import java.applet.*;

/* 
The class that provides interface between Tango system and user application. In IDE it is
represented by navy blue 3D rectangle with "TANGO" label. Tango bean conforms to Java Beans
standard so that it can be used and manipulated visually in design environments. 
*/

public class TangoBean extends Component implements java.io.Serializable, PipeConnectListener, 
TControlListener, TDataListener
{
    
    // Tango bean color
    private Color componentColor = Color.blue;
    
    // Tango bean text color
	private Color textColor = Color.black; 
  
    // Table of Tango events listeners, used to distribute data events
    // contains references to Pipe objects with pipe (channel) number as the key
    private Hashtable tango_listeners = new Hashtable();
    
    // Table of Tango events listeners, used to distribute data events
    // contains references to NamedPipe objects with pipe target name as the key
    private Hashtable named_listeners = new Hashtable();
    
    // Vector of Tango control listeners i.e. components interested in
    // Tango control events such as change of the master or participant list change
    private Vector master_listeners = new Vector(); 
    
    // Vector of Tango control listeners i.e. components interested in
    // Tango control events such as change of the master or participant list change
    private Vector participants_listeners = new Vector(); 
    
    // Vector of Tango control listeners i.e. components interested in
    // Tango control events such as change of the master or participant list change
    private Vector audio_listeners = new Vector(); 
    
    // Used to generate pipe (channel) id
    private int id = 0;  
    
    // Text pads used do calculate preferred size of the Tango bean. Define space
    // that should be left around "TANGO" string in the rectangle
    static final int TEXT_XPAD = 10;
    static final int TEXT_YPAD = 10;  
    
    // Used to interact with Tango system
    private TAgent tango;
    
    // In Tango/Out of Tango mode switch. True means that application and Tango bean work 
    // in Tango system
    private boolean inTango;
    
    /* 
    Creates new Tango bean    
    */
    
	public TangoBean()
	{
		super();
		
		// set outside Tango mode as default
		inTango = false;
		tango = null;
		
		// set font and resize
		setFont(new Font("Dialog", Font.PLAIN, 12));
		sizeToFit();																	  																	  	   
						
		// make Tango bean visible in design time and invisible in run time				
						
		if (Beans.isDesignTime()) {		    
		    setVisible(true);    
		} else {		    
		    setVisible(false); 	
		}		
	}

	//********************** Graphics methods used by IDE to draw Tango bean ************
		
	/*
	Repaints Tango bean after modifications	
	*/

	public synchronized void paint(Graphics context)
	{
	    int width = getSize().width;
    	int height = getSize().height;
	    
	    // retrieve metrics
	    
	    FontMetrics fm = getFontMetrics(getFont());
	    
	    // draw 3D rectangle
		context.setColor(componentColor);
		context.fill3DRect(0,0, width, height, true);
		context.setColor(textColor);
		
		// draw "TANGO" string in the middle of the rectangle
		context.drawString("TANGO", (width - fm.stringWidth("TANGO")) / 2, (height + fm.getMaxAscent() - fm.getMaxDescent()) / 2);
		context.setColor(componentColor);
	}	
	
	/*
	Returns preferred size of the Tango bean	
	*/
	
	public Dimension getPreferredSize() {
	FontMetrics fm = getFontMetrics(getFont());
	//System.out.println("current size " + getSize().width + " " + getSize().height); 
	//System.out.println("preferred size " + (fm.stringWidth("TANGO") + 2* TEXT_XPAD) + " " + (fm.getMaxAscent() + fm.getMaxDescent() + 2*TEXT_YPAD));
	
	// preferred size is calculated using "TANGO" string, it must fit into rectangle
	
	return new Dimension(fm.stringWidth("TANGO") + 2* TEXT_XPAD, 
			     fm.getMaxAscent() + fm.getMaxDescent() + 2*TEXT_YPAD);
    }

    /**
     * @deprecated provided for backward compatibility with old layout managers.
     */
    public Dimension preferredSize() {
        
    //    System.out.println("preferred size called " + getSize().width + " " + getSize().height); 
	    return getPreferredSize();
    }
    
    /*
    Returns minimum size    
    */
    public Dimension getMinimumSize() {
    	return getPreferredSize();
    }
    
    /**
     * @deprecated provided for backward compatibility with old layout managers.
     */
    public Dimension minimumSize() {
	    return getMinimumSize();
    }

    /*
    Sets size of Tango bean so that "TANGO" string is visible    
    */
    
    private void sizeToFit() {
	
    	Dimension pd = getPreferredSize();
    	setSize(pd.width, pd.height);
//    	System.out.println("sizetofit size " + getSize().width + " " + getSize().height); 

        // force parent to redraw Tango bean        
    	Component p = getParent();
    	if (p != null) {
	        p.invalidate();
	        p.validate();
	     }
	
    }               
    
    //************************** Tango specific methods **********************************
    
    /*
    Initializes Tango bean to work in Tango mode. This method must be called by application
    to start interaction with Tango environment using Tango bean.    
    */
    
    public void tangoInit(TLAgent tla) throws TangoException {
        		        
       // try {
            
            // create TAgent object that talks directly to Tango
            tango = new TAgent(tla);
            
            // register Tango bean in TAgent object to receive data, control and cross data 
            // messages
            
    		tango.addTDataListener(this);
    		tango.addTControlListener(this);
//    		tango.addTCrossListener(this);
    		
    		// set Tango bean to work in Tango mode
    		inTango = true;
    		        
       // } catch(Exception e) {
    		        
    //		System.out.println("TangoBean: not in Tango");
     //   }	               
    }
    
    /*
    Informs Tango bean that application terminates. Tango bean informs Tango system about this
    event and Tango removes application from the session    
    */
    
    public void tangoExit() {
        
        // inform Tango about application termination        
        if (inTango) {            
              tango.exit();
        }               
    }
   
    
    //************************** Tango control events handling **********************************************
    
    /*
    Add and remove ControlListener
    */

    public synchronized void addMasterChangeListener(MasterChangeListener sl) {
         master_listeners.addElement(sl);      
    }

    public synchronized void removeMasterChangeListener(MasterChangeListener sl) {
        master_listeners.removeElement(sl);
    }   
    
    public synchronized void addParticipantsChangeListener(ParticipantsChangeListener sl) {
         participants_listeners.addElement(sl);      
    }

    public synchronized void removeParticipantsChangeListener(ParticipantsChangeListener sl) {
        participants_listeners.removeElement(sl);
    }   
    
    public synchronized void addAudioRequestListener(AudioRequestListener sl) {
         audio_listeners.addElement(sl);      
    }

    public synchronized void removeAudioRequestListener(AudioRequestListener sl) {
        audio_listeners.removeElement(sl);
    }   
    
    
    // TControlListener implementation. Following methods are called by TAgent when state of
    // the Tango system changes. In response to this action Tango bean sends TangoControlEvent
    // to all registered control listers.
    
    
    /*
    Session master changed    
    */    
    public void masterChanged(boolean isMaster, String masterName) {
        
      MasterChangeEvent tce;		
	  tce = new MasterChangeEvent(this, isMaster, masterName);        

      Vector targets;
	  synchronized (this) {

	       targets = (Vector) master_listeners.clone();
	  }		

      // send TangoControlEvent to all interested components
	  for (int i = 0; i < targets.size(); i++) {

	        MasterChangeListener target = (MasterChangeListener)targets.elementAt(i);
	        //call corresponding method in ControlListener
	        target.masterChanged(tce);
	  }                  
    }

    /*
    New participant entered session. State of the participant list changed    
    */
    public void participantJoined(String name){
    
      ParticipantsChangeEvent tce;		
	  tce = new ParticipantsChangeEvent(this, name);        

      Vector targets;
	  synchronized (this) {

	       targets = (Vector) participants_listeners.clone();
	  }		
	  
      // send TangoControlEvent to all interested components
	  for (int i = 0; i < targets.size(); i++) {

	        ParticipantsChangeListener target = (ParticipantsChangeListener)targets.elementAt(i);
	        //call corresponding method in ControlListener
	        target.participantJoined(tce);
	  }        
        
    }
    /*
    Participant left session. State of the participant list changed    
    */
    public void participantLeft(String name){
    
      ParticipantsChangeEvent tce;		
	  tce = new ParticipantsChangeEvent(this, name);        

      Vector targets;
	  synchronized (this) {

	       targets = (Vector) participants_listeners.clone();
	  }		
	  
      // send TangoControlEvent to all interested components
	  for (int i = 0; i < targets.size(); i++) {

	        ParticipantsChangeListener target = (ParticipantsChangeListener)targets.elementAt(i);
	        //call corresponding method in ControlListener
	        target.participantLeft(tce);
	  }        
        
    }
  
    public void audioRequested(boolean request){
     
         AudioRequestEvent tce;		
	     tce = new AudioRequestEvent(this, request);        

         Vector targets;
	     synchronized (this) {

	       targets = (Vector) audio_listeners.clone();
	     }		
        // send TangoControlEvent to all interested components
    	  for (int i = 0; i < targets.size(); i++) {
	        AudioRequestListener target = (AudioRequestListener)targets.elementAt(i);	        
	         //call corresponding method in ControlListener
	        target.audioChanged(tce);
	      }                 
     }  
    
    //End of ControlListener interface implementation     
    
    
    //********************************** Access to the Tango control information ********************
    
    /*
    Checks master/slave status of the application. If 'true' value is returned, the application is
    a master of the session
    */
    
    public boolean isMaster() {
        
      if (inTango) {  
        
            return tango.isMaster();          
      } else {
        
      //  System.out.println("isMaster");
        return false;          
      }
    }    

    /*
    Retrieves name of the user that started the application    
    */
    public String getUserName() {
        
      if (inTango) {        
            return tango.getUserName();
      } else {              
          //System.out.println("getUserName");
          return "test_user_name";
      }
    }

    /*
    Retrieves the name of the current session master
    */    
    public String getMasterName()
 
    {           
      if (inTango) {
        
            return tango.getMasterName();
      } else {        
        
          //System.out.println("getMasterName");
          return "test_master_name";
      }
    }    
    
    /*
    Retrieves list of the participants of the session the application is in
    */

    public String[] getParticipantNames() {                          
        
      if (inTango) {  
        
            return tango.getParticipantNames();  
        
      } else {
        
        String[] testTable = new String[2];
        
        testTable[0] = "test_participant_1";
        testTable[1] = "test_participant_2";
          //System.out.println("getParticipants");
          return testTable;          
      }
    }  
    
    public boolean isAudioAvailable() {
        
      if (inTango) {  
        
            return tango.isAudioAvailable();          
      } else {
        
      //  System.out.println("isAudioAvailable");
        return true;          
      }
    }    
    
          	
	//************************ Communication with pipes ******************************

    // PipeConnectListener interface implementation

	/*
	Registers new pipe at Tango bean. Tango bean is a target of PipeConnectEvent because it
	implements PipeConnectListener. Whenever new pipe is to be added, the pipe must send 
	the event to Tango bean using PipeConnectLister interface. 	
	*/
	
    public synchronized void pipeReady(PipeConnectEvent cre) {
       
       
      // if (cre.fromNamedPipe()) {
         // retrieve sender 
         //  NamedPipe pp = (NamedPipe) cre.getSource();        
       
         //  String name = pp.getTargetName();
       
         // if (name == null) { 
        
         //      System.out.println("TangoBean: name for named pipe is not specified");
         //     return;
         // }       
       
         // if (named_listeners.containsKey(name)) {
        
         //   System.out.println("TangoBean: named pipe " + name + " already connected");
         //  return;        
         // }
       
           // set this Tango bean as a message receiver from the pipe
         //  pp.setControl(-1, this);
       
          // add new pipe to the table of message listeners with specific name
         // System.out.println("TangoBean: pipe added with name " + name);       
        //  named_listeners.put(name, pp);                       
        
    //   } else {
       
           // retrieve sender 
           Pipe pp = (Pipe) cre.getSource();        
       
           // set this Tango bean as a message receiver from the pipe
           pp.setControl(id, this);
       
           // add new pipe to the table of message listeners with specific id
           tango_listeners.put(new Integer(id), pp);   
       
           // assure that id is unique
           id ++;            
     //  }    
    
    }
    
    // End of PipeConnectListener interface implementation

    /*
    Called by Pipe object. Sends message to the corresponding Pipe objects with the same id 
    in all applications that are in Tango session. If Tango bean is in out of Tango mode,
    the message is looped back and returned to the sender pipe    
    */
    public void sendToTango(int channelID, Object toTango) throws IOException {

        //System.out.println("tango bean : message sent to tango");
        
        if(inTango) {
            
              // encode data and send message to Tango
              tango.send(encode(channelID, toTango));
            
        } else {
            
              // encode data and loop back message to the sender  
              receive(encode(channelID, toTango));                           
        }
    }
    
     /*
    Called by Pipe object. Sends message to the corresponding Pipe objects with the same id 
    in all applications that are in Tango session. If Tango bean is in out of Tango mode,
    the message is looped back and returned to the sender pipe    
    */
    public void selectiveSendToTango(String[] participants, int channelID, Object toTango) throws IOException {

        //System.out.println("tango bean : message sent to tango");
        
        if(inTango) {
            
              // encode data and send message to Tango
              tango.selectiveSend(participants, encode(channelID, toTango));
            
        } else {
            
              // encode data and loop back message to the sender  
              receive(encode(channelID, toTango));                           
        }
    }
    
    
    // TDataListener interface implementation
    
    /*
    Called by TAgent object. Signals arrival of new message from Tango. Tango bean resends
    the message to appropriate pipe.    
    */
    public void receive(byte data[]) { 
        
//        System.out.println("tango bean: message received from tango");        
    	Object content[];			
        Pipe pp;        
	
	    // decode data
	    
	    try { 
        	content = decode(data);
        
    	
    	// get reference to proper pipe
            pp = (Pipe) tango_listeners.get(content[0]);
        
        // send message to the pipe
            pp.receiveData(content[1]); 
        } catch(Exception e) {
            
            e.printStackTrace();   
        }
        
    }
    
    // End of TDataListener interface implementation
 
/*    public void crossSend(String sender, Object data) {
        
        if(inTango) {
            
              // encode data and cross send message to other application
              tango.crossSend(sender, encode(-1, data));
            
        } else {
            
              // encode data and loop back message to the sender  
              crossReceive(sender, encode(-1, data));                           
        }                    
    } */
 
    // TCrossListener interface implementation
 
/*    public void crossReceive(String sender, byte data[]){
        
        Object content[];			
        NamedPipe pp;        
	
	    // decode data
    	content = decode(data);
    	
    	System.out.println("TangoBean: sender " + sender);
    	// get reference to proper pipe
        pp = (NamedPipe) named_listeners.get(sender);
        
        // send message to the pipe
        pp.receiveData(content[1]);           
        
    }  */
    
    // End of TCrossListener interface implementation
    
    /*
    Encodes message to be sent using Tango framework. This method takes id and object as 
    parameters and returns table of bytes that can be sent using Tango framework.    
    */

    private byte[] encode(int id, Object obj) throws IOException {

            ByteArrayOutputStream baos;
            ObjectOutputStream oos;

//            try {

                baos = new ByteArrayOutputStream();
                oos = new ObjectOutputStream(baos);
                
                // write id and data into byte array stream
	        	oos.writeInt(id);
                oos.writeObject(obj);

//            } catch(Exception e) {

//                e.printStackTrace();
//                baos = null;
//	            return null;
//            }

            // return table of bytes            
            return baos.toByteArray();
        }         

    /*
    Decode table of bytes received from Tango. Returned table of objects contains sender pipe
    id and data.
    */
    private Object[] decode(byte table[]) throws IOException, ClassNotFoundException {

            ByteArrayInputStream bais;
            ObjectInputStream ois;
            Object result[] = new Object[2];
	        Object data;
            int id;
	 
//            try {
                bais = new ByteArrayInputStream(table);
                ois = new ObjectInputStream(bais);
	
	            // retrieve id and data
        		id = ois.readInt();
		        data = ois.readObject();
				
				// fill Object table
        		result[0] = new Integer(id);
		        result[1] = data;
	
//	       } catch(Exception e) {

//                e.printStackTrace();	    
//                return null;
//           }
          
           return result;
    }    
    
    // To be removed. Used for compatibility with  older versions      
 //   public void receiveFromTango(byte data[]) {}
    
    // To be removed. Used for compatibility with  older versions
//    public void receiveControlFromTango(byte data[]) {}    
                             
}
