/***************************************************************************
  FILE: TLAgentFake.java, TomaszMajor-10May98
  PURPOSE: use with TServerFake
  $Id: TLAgentFake.java,v 1.7 1999/04/22 21:47:37 toma Exp $
  Component: Plugin, Interfaces
***************************************************************************/


package webwisdom.tango.fake;


import java.net.*;
import java.io.*;
import java.awt.*;
import java.util.Vector;

import webwisdom.tango.TangoException;
import webwisdom.tango.TLAgent;
import webwisdom.tango.TLListener;
import webwisdom.tango.messages.AppEventMessage;
import webwisdom.tango.messages.CAppAnswerEventMessage;
import webwisdom.tango.consts.Const;
import webwisdom.tango.consts.CAConsts;

import netscape.security.PrivilegeManager;


class TLAgentFakeReceiver extends Thread
{
    private static final String CL="TLAgentFakeReceiver";


    DataInputStream inStream;
    TLAgentFake client;


    public TLAgentFakeReceiver(DataInputStream is,TLAgentFake tc)
    {
      //System.out.println(CL+".TLAgentFakeReceiver("+is+","+tc+")");
      inStream=is;
      client=tc;
    }// TLAgentFakeReceiver ________________________________________________


    public void run()
    {
      //System.out.println(CL+".run()");
      try
      {
	do
	{
	  byte type=inStream.readByte();
	  if(type==TServerFake.DATA)
	  {
	    int packetLen=inStream.readInt();
	    byte[] packetData=new byte[packetLen];
	    inStream.readFully(packetData);
	    client.receive(packetData);
	    //System.out.println(CL+": < DATA["+packetLen+"]");
	  }
	  else if(type==TServerFake.MASTER)
	  {
	    //System.out.println(CL+": < MASTER");
	    client.setMaster(true);
	  }
	  else if(type==TServerFake.SLAVE)
	  {
	    System.out.println(CL+": < SLAVE");
	    client.setMaster(false);
	  }
	}
	while(true);
      }
      catch(EOFException e)
      {
	System.err.println(CL+": Server disconnected!");
      }
      catch(IOException e)
      {
	System.err.println(CL+": "+e);
      }
    }// run ________________________________________________________________


    public String toString()
    {
	return CL+"[]";
    }// toString ___________________________________________________________


}// TLAgentFakeReceiver ////////////////////////////////////////////////////




class CAppAnswerEventMessageFake extends CAppAnswerEventMessage
{
    private static final String CL="CAppAnswerEventMessageFake";


    private byte type;
    private TLAgentFake agent;


    CAppAnswerEventMessageFake(AppEventMessage m,TLAgentFake a)
    {
      super(new byte[1]);
      type=m.getData()[0];
      agent=a;
    }// CAppAnswerEventMessageFake _________________________________________


    public byte getAnswerType()
    {
      return type;
    }
    

    public boolean isMaster()
    {
      return getMastername().equals(getUsername());
    }


    public String getMastername()
    {
      return agent.masterName;
    } 
    

    public String getUsername()
    {
      //return (String)(getParticipants().firstElement());
      return agent.userName;
    }
    

    public Vector getParticipants()
    {
      return agent.participantNames;
    }


    public Vector getActiveUsers()
    {
      return getParticipants();
    }


    public Vector getLocalApps()
    {
      return new Vector();
    }


    public String getHostname()
    {
      return "host";
    } 


    public String toString()
    {
      return CL+"[]";
    }// toString ___________________________________________________________


}// CAppAnswerEventMessageFake /////////////////////////////////////////////




/**This class can be used instead of TangoAgent
   for browserless testing of Tango applets with FakeTangoServer.
   *@see FakeTangoServer
   *@author Tomasz Major*/
public class TLAgentFake implements TLAgent
{
    private static final String CL="TLAgentFake";

    
    private TLListener lowListener=null;


    private Socket socket;
    private DataOutputStream outStream;
    private DataInputStream inStream;

    private TLAgentFakeReceiver receiverThread;


    String userName="user";
    Vector participantNames=new Vector();
    String masterName="master";//new String(userName);


    /**Creates TLAgent which connects to TServerFake.
     *@param serverHost,serverPort fake tango server
     *@exception TangoException if cannot connect to the server*/
    public TLAgentFake(String serverHost,int serverPort)
      throws TangoException
    {
      //System.out.println(CL+".TLAgentFake("+serverHost+","+serverPort+")");
      netscape.security.PrivilegeManager.enablePrivilege("UniversalConnect");
      try
      {
	socket=new Socket(serverHost,serverPort);
	outStream=new DataOutputStream(socket.getOutputStream());
	inStream=new DataInputStream(socket.getInputStream());
	System.out.println(CL+": connected to: "+socketToString(socket)+
			   " (localPort="+socket.getLocalPort()+")");
      }
      catch(IOException e)
      {
	throw new TangoException(serverHost+":"+serverPort+" unreachable");
      }
      //System.out.println(CL+": socket="+socket+",inStream="+inStream+",outStream="+outStream);


      setSessionAttributes("ben","jerry",null);


      receiverThread=new TLAgentFakeReceiver(inStream,this);
      //receiverThread.setPriority(Thread.MIN_PRIORITY);
      receiverThread.start();
      (Thread.currentThread()).setPriority(Thread.MIN_PRIORITY);
    }// TLAgentFake ________________________________________________________


    /**
     *@param u this user name
     *@param m name of the session master
     *@param us names of other users in the session*/
    public void setSessionAttributes(String u,String m,String[] us)
    {
      //System.out.println(CL+".setSessionAttributes("+u+","+m+","+us+")");

      userName=u;
      masterName=(m==null)?new String(u):m;
      //masterName="maestro";

      boolean uAdded=false,mAdded=false;
      Vector userNames=new Vector();
      if(us!=null)
      {
	for(int i=0;i<us.length;i++)
	{
	  String ui=us[i];
	  if(ui.equals(userName))
	    uAdded=true;
	  else if(ui.equals(masterName))
	    mAdded=true;
	  userNames.addElement(ui);
	}
      }
      if(!uAdded)
	userNames.addElement(u);
      if(!mAdded)
	userNames.addElement(masterName);
      //System.out.println(CL+".setSessionAttributes(): userName="+userName+",masterName="+masterName);
    }// setSessionAttributes _______________________________________________


    /**
     *@deprecated*/
    public TLAgentFake(String userName,boolean isMaster,
		       String serverHost,int serverPort)
    {
      //System.out.println(CL+".TLAgentFake("+serverHost+","+serverPort+")");
      System.err.println(CL+".TLAgentFake(String,boolean,String,int): deprecated!");
try
 {
      try
      {
	socket=new Socket(serverHost,serverPort);
	outStream=new DataOutputStream(socket.getOutputStream());
	inStream=new DataInputStream(socket.getInputStream());
	System.out.println(CL+": connected to: "+socketToString(socket)+
			   " (localPort="+socket.getLocalPort()+")");
      }
      catch(IOException e)
      {
	e.printStackTrace();
	return;
      }


      String[] us={"ben","jerry"};
      setSessionAttributes(userName,isMaster?userName:userName+"Master",us);


      receiverThread=new TLAgentFakeReceiver(inStream,this);
      //receiverThread.setPriority(Thread.MIN_PRIORITY);
      receiverThread.start();
      (Thread.currentThread()).setPriority(Thread.MIN_PRIORITY);
 }
catch(Exception ee)
 {
   ee.printStackTrace();
 }
    }// TLAgentFake ________________________________________________________


    public void exit()
    {
      receiverThread.stop();
      try
      {
	inStream.close();
	outStream.close();
	socket.close();
      }
      catch(IOException e)
      {
      }
    }// exit _______________________________________________________________


    public void setTLListener(TLListener l)
    {
      lowListener=l;
    }// setTLListener ______________________________________________________


    public void send(int i[],AppEventMessage m)
    {
    }// send _______________________________________________________________

  
    public void send(AppEventMessage m)
    {
      try
      {
	if(m.getType()==Const.CONTROL)
	{
	  if(lowListener!=null)
	    lowListener.receive(new CAppAnswerEventMessageFake(m,this));
	}
	else //if(ok)
	{
	  byte[] b=m.getData();
	  outStream.writeInt(b.length);
	  outStream.write(b);
	  outStream.flush();
	  //System.out.println(CL+".send(byte["+b.length+"])");
	}
      }
      catch(IOException e)
      {
	System.err.println(CL+".send(): "+e);
      }
    }// send _______________________________________________________________


    /*interface for TLAgentFakeReceiver*/
    void receive(byte[] b)
    {
      if(lowListener!=null)
	lowListener.receive(new AppEventMessage(Const.DATA,b));
    }// receive ____________________________________________________________


    /*interface for TLAgentFakeReceiver*/
    void setMaster(boolean isM)
    {
      //System.out.println(CL+".setMaster("+isM+")");
      if(isM)
	masterName=userName;
      if(lowListener!=null)
      {
	byte[] b=new byte[2];
	b[0]=CAConsts.ISMASTER;
	b[1]=isM?CAConsts.YES:CAConsts.NO;
	CAppAnswerEventMessage m=new CAppAnswerEventMessage(b);
	//System.out.println(CL+".setMaster(): "+m);
	lowListener.receive(m);
      }
    }// receive ____________________________________________________________


    public int getSessionId()
    {
      return -1;
    }// getSessionId _______________________________________________________


    /*utility*/
    public static String socketToString(Socket s)
    {
      return "["+s.getInetAddress().getHostName()+":"+s.getPort()+"]";
    }// socketToString _____________________________________________________


    public String toString()
    {
      String host=socket.getInetAddress().getHostName();
      int port=socket.getPort();
      return CL+"["+host+":"+port+"]";
    }// toString ___________________________________________________________


}// TLAgentFake ////////////////////////////////////////////////////////////


