/***************************************************************************
  FILE: TServerFake.java, TomaszMajor-10May98
  PURPOSE: easy testing of Tango applets
  $Id: TServerFake.java,v 1.3 1999/03/30 04:37:26 toma Exp $
  Component: Interfaces
***************************************************************************/


package webwisdom.tango.fake;


import java.net.*;
import java.io.*;
import java.util.Vector;
import java.util.NoSuchElementException;

import java.awt.*;


class ClientThread extends Thread
{	
    private static final String CL="ClientThread";
    private static final String CL2="FTS";


    TServerFake server;

    Socket socket;
    private String host;
    private int port;
    DataInputStream inStream;
    DataOutputStream outStream;


    public ClientThread(TServerFake s,Socket so) throws IOException
    {
      server=s;
      socket=so;
      host=socket.getInetAddress().getHostName();
      port=socket.getPort();
      inStream=new DataInputStream(socket.getInputStream());
      outStream=new DataOutputStream(socket.getOutputStream());
      System.out.println(CL2+"."+this+": accepted");
    }// ClientThread _______________________________________________________


    public void run()
    {
      synchronized(server.clients)
      {
	try //send master or slave message
	{
	  DataOutputStream s=outStream;
	  if(server.master==null)
	  {
	    System.out.println(CL2+"."+this+": becomes master");
	    s.writeByte(TServerFake.MASTER);
	    s.flush();
	    server.master=this;
	  }
	  else
	  {
	    System.out.println(CL2+"."+this+": becomes slave");
	    s.writeByte(TServerFake.SLAVE);
	    s.flush();
	  }
	}
	catch(IOException e)
	{
	  System.err.println(CL2+"."+this+": disconnecting "+this);
	  server.clients.removeElement(this);
	  return;
	}
      }

      do
      {
	int packetLen=0;
	byte[] packetData=null;
	try
	{
	  packetLen=inStream.readInt();
	  packetData=new byte[packetLen];
	  inStream.readFully(packetData);
	  //System.out.println(CL+".Received: "+packetLen);
	}
	catch(EOFException e)
	{
	  System.err.println(CL2+"."+this+": disconnecting");
	  synchronized(server.clients)
	  {
	    server.clients.removeElement(this);
	    if(this==server.master)
	      findMaster();
	  }
	  stop2();
	  return;
	}
	catch(IOException e)
	{
	  System.err.println(CL2+"."+this+": !ignoring: "+e);
	  //e.printStackTrace();
	  continue;
	}
	
	synchronized(server.clients)
	{
	  boolean noMaster=false;
	  for(int i=0;i<server.clients.size();i++)
	  {
	    ClientThread cl=(ClientThread)server.clients.elementAt(i);
	    //System.out.println(CL+".Serving["+i+"]: "+cl);
	    if(cl==this)
	      continue;
	    try
	    {
	      DataOutputStream s=cl.outStream;
	      s.writeByte(TServerFake.DATA);
	      s.writeInt(packetLen);
	      s.write(packetData);
	      s.flush();
	      //System.out.println(CL+".Sent["+i+"]: "+packetLen);
	    }
	    catch(IOException e)
	    {
	      System.err.println(CL2+"."+this+": disconnecting "+cl);
	      cl.stop2();
	      server.clients.removeElement(cl);
	      if(cl==server.master)
		noMaster=true;
	      i--;
	    }
	  }
	  if(noMaster)
	    findMaster();
	}
      }
      while(true);
    }// run ________________________________________________________________


    private void findMaster()
    {
      try
      { //candidate for new master
	ClientThread cl=(ClientThread)server.clients.firstElement();
	try //send master message to candidate master
	{
	  System.out.println(CL2+"."+cl+": becomes slave");
	  DataOutputStream s=cl.outStream;
	  s.writeByte(TServerFake.MASTER);
	  s.flush();
	  //System.out.println(CL+".Sent["+i+"]: "+packetLen);
	  server.master=cl;
	}
	catch(IOException e)
	{
	  System.err.println(CL2+"."+this+": disconnecting "+cl);
	  cl.stop2();
	  server.clients.removeElement(cl);
	  findMaster(); //try again with reduced set of clients
	}
      }
      catch(NoSuchElementException e) //no more clients
      {     
	server.master=null;
      }
    }// find Master ________________________________________________________


    public void stop2()
    {
      try
      {
	inStream.close();
	outStream.close();
	socket.close();
      }
      catch(IOException e)
      {
      }
      stop();
    }// stop2 ______________________________________________________________


    public String toString()
    {
      return "["+host+":"+port+"]";
    }// toString ___________________________________________________________


    public static String socketToString(Socket s)
    {
      return "["+s.getInetAddress().getHostName()+":"+s.getPort()+"]";
    }// socketToString _____________________________________________________


}// ClientThread ///////////////////////////////////////////////////////////




/**Simple server broadcasting messages from clients to all other clients.
   Clients are instances of FakeTangoAgent.
   Run with parameter being a port number.
   Console communicates have two formats:<ul>
   <li><b>FTS: </b><i>message</i>; or
   <li><b>FTS.[</b><i>clientThread</i><b>]: </b><i>message</i>.</ul>
   *@see FakeTangoAgent
   *@author Tomasz Major*/
public class TServerFake
{
    private static final String CL="TServerFake";
    private static final String CL2="FTS";


    ServerSocket server;
    static final int MAX_CLIENTS=10;
    Vector clients=new Vector(MAX_CLIENTS,1);
    ClientThread master=null;


    public static final byte DATA=17;
    public static final byte MASTER=18;
    public static final byte SLAVE=19;


    /**Anchors at the given port.
     *@param port port at which the server is to listen for clients
     *@exception IOException if cannot start service at given port*/
    public TServerFake(int port) throws IOException
    {
      server=new ServerSocket(port);
      port=server.getLocalPort();
      System.out.println(CL2+": started on port "+port);
    }// TServerFake ________________________________________________________


    /**Starts the server.
       Servers accepts connections from clients
       and creates new thread for each client.
       *@exception IOException not caught system exceptions*/
    public void run() throws IOException
    {
      do
      {
	Socket socket=server.accept();
	if(clients.size()>=MAX_CLIENTS)
	{
	  System.err.println(CL2+": rejecting client "+
			     ClientThread.socketToString(socket));
	  socket.close();
	  continue;
	}
	ClientThread cl=new ClientThread(this,socket);
	cl.setPriority((Thread.currentThread()).getPriority()+1);
	synchronized(clients)
	{
	  cl.start();
	  clients.addElement(cl);
	}
      }
      while(true);
    }// run ________________________________________________________________


    public String toString()
    {
	return CL+"[#="+clients.size()+"]";
    }// toString ___________________________________________________________


    /**Creates and starts the server at the port given the first parameter.*/
    public static void main(String[] args)
    {
      int port=-1;
      try
      {
	port=Integer.valueOf(args[0]).intValue();
      }
      catch(NumberFormatException e)
      {
	System.err.println(CL+".main(): Usage: main(port)");
	System.exit(2);
      }

      TServerFake s=null;
      try
      {
	s=new TServerFake(port);
      }
      catch(IOException e)
      {
	System.err.println(CL+".main(): Cannot start server at port "+port+": "+e);
	System.exit(1);
      }

      try
      {
	s.run();
      }
      catch(IOException e)
      {
	System.err.println(CL+".main(): I/O error: "+e);
	System.exit(1);
      }
    }// main _______________________________________________________________


}// TServerFake ////////////////////////////////////////////////////////////


