// $Id: CommandMessage.java,v 1.4 1998/07/10 20:15:22 beca Exp $
/*
File: CommandMessage.java
Component: Server, Plugin
Date		Author		Changes
Aug 22 96	Lukasz Beca	Created
*/

package webwisdom.tango.messages;

import webwisdom.tango.messages.Message;
import webwisdom.tango.consts.Const;
import java.io.*;


/**
 * A class representing message COMMAND
 * 
 *@author Lukasz Beca
 *@see Central Server - Local Demon protocol
 **/
public class CommandMessage extends Message
{
    private static final String CL="CommandMessage";


    private int AT;
    private String PARAM;
    private int AID;
    private int SID;


    /**Creates a new COMMAND message
     *@param ATparam  application type
     *@param PARAMparam  parameters for application
     *@param AIDparam application number
     *@param SIDparam  session number*/
    public CommandMessage(int ATparam, String PARAMparam,
			  int AIDparam, int SIDparam)
    {
      AT = ATparam;
      SID = SIDparam;
      PARAM = new String(PARAMparam);
      AID = AIDparam;
      type = Const.COMMAND;
    }


    /**Creates a new COMMAND message, information is read from the input stream
     *@param instream  input stream
     *@exception IOException*/
    CommandMessage(DataInputStream instream) throws IOException
    {
      AT = instream.readInt();
      PARAM = instream.readUTF();
      AID = instream.readInt();
      SID = instream.readInt();
      type = Const.COMMAND;
    }


    /**Writes COMMAND message to the output stream
     *@param outstream  output stream*/
    void send(DataOutputStream outstream) throws IOException
    {
      // System.out.println("Sending type " + Const.COMMAND);
      outstream.writeInt(Const.COMMAND);
      outstream.writeInt(AT);
      outstream.writeUTF(PARAM);
      outstream.writeInt(AID);
      outstream.writeInt(SID);
      outstream.flush();
    }
  

    /**
     *@return application type*/
    public int getAT()
    {
      return AT;
    }  


    /**
     *@return session number*/
    public int getSID()
    {
      return SID;
    }


    /**
     *@return parameters*/
    public String getPARAM()
    {
      String result;
      result =  new String(PARAM);
      return result;
    }


    /**
     *@return application number*/
    public int getAID()
    {
      return AID;
    }
  

    public String toString()
    {
      return CL+"[AT="+AT+",PARAM="+PARAM+",AID="+AID+",SID="+SID+"]";
    }// toString ___________________________________________________________


}// CommandMessage /////////////////////////////////////////////////////////


