// $Id: JaskMessage.java,v 1.4 1998/07/10 20:15:27 beca Exp $
/*
File: JaskMessage.java
Component: Server, Plugin
Date		Author		Changes
Aug 24 96	Lukasz Beca	Created
*/
package webwisdom.tango.messages;

import java.io.*;
import webwisdom.tango.consts.Const;
import webwisdom.tango.messages.Message;

/**
 * A class representing message JASK
 * 
 *@author Lukasz Beca
 *@see Central Server - Local Demon protocol
 **/

public class JaskMessage extends Message
{
    private static final String CL="JaskMessage";

  
    private int UID;
    private int SID;
    private int AT;


 /**
   * Creates a new JASK message
   *@param ATpar application type
   *@param SIDpar session number
   *@param UIDpar user number
   **/

  public JaskMessage(int ATpar, int SIDpar, int UIDpar) {

    AT = ATpar;
   SID = SIDpar;
   UID = UIDpar;
  
   type = Const.JASK;
  }

  /**
   * Creates a new JASK message, information is read from the input stream
   *@param instream  input stream
   **/

  JaskMessage(DataInputStream instream) throws IOException {
    
    
    AT = instream.readInt();
    SID= instream.readInt();     
    UID = instream.readInt();
       
    type = Const.JASK;
    
  }

 /**
   * Writes JASK message to the output stream
   *@param outstream  output stream
   **/

  void send(DataOutputStream outstream)  throws IOException {
      
    //System.out.println("Sending type " + Const.JASK);
    outstream.writeInt(Const.JASK);
    //System.out.println("Sending AT " + AT); 
    outstream.writeInt(AT);
    //System.out.println("Sending SID " + SID);
    outstream.writeInt(SID);
    //System.out.println("Sending UID " + UID);
    outstream.writeInt(UID);
     
    outstream.flush();
   
  }
  

 /**
   * Returns user number
   **/

  public int getUID() {
       
    return UID;

  }  

  /**
   * Returns session number
   **/

  public int getSID() {
   
    return SID;
  }


  /**
   * Returns application type
   **/

  public int getAT() {
   
    return AT;
  }
  
    public String toString()
    {
      return CL+"[AT="+AT+",SID="+SID+",UID="+UID+"]";
    }// toString __________________________________________________________


}// JaskMessage ///////////////////////////////////////////////////////////

