// $Id: RaskMessage.java,v 1.4 1998/07/10 20:15:36 beca Exp $
/*
File: RaskMessage.java
Component: Server, Plugin
Date		Author		Changes
Aug 22 96	Lukasz Beca	Created
*/


package webwisdom.tango.messages;


import webwisdom.tango.messages.Message;
import webwisdom.tango.consts.Const;
import java.io.*;


/**
 * A class representing RASK message
 * 
 *@author Lukasz Beca
 *@see Central Server - Local Demon protocol
 **/
public class RaskMessage extends Message
{
    private static final String CL="RaskMessage";


    private int AT;
    private int SID;
    private int localUID;


 /**
   * Creates a new RASK message
   *@param ATpar application type
   *@param SIDpar session number
   *@param localUIDpar local user number
   **/

  public RaskMessage(int ATparam, int SIDparam, int localUIDpar) {

   AT = ATparam;
   SID = SIDparam;
   localUID = localUIDpar;

   type = Const.RASK;
  }

 /**
   * Creates a new RASK message, information is read from the input stream
   *@param instream  input stream
   **/

  RaskMessage(DataInputStream instream) throws IOException {
    

    AT = instream.readInt();    
    SID = instream.readInt();
    localUID = instream.readInt();
    
    type = Const.RASK;
    
  }

 /**
   * Writes RASK message to the output stream
   *@param outstream  output stream
   **/

  void send(DataOutputStream outstream) throws IOException {
  
   
    //System.out.println("Sending type " + Const.RASK);
    outstream.writeInt(Const.RASK);
    //System.out.println("Sending AT " + AT);
    outstream.writeInt(AT);     
    //System.out.println("Sending SID " + SID);
    outstream.writeInt(SID);
    //System.out.println("Sending localUID " + localUID);
    outstream.writeInt(localUID);

    outstream.flush();
   
  }

 /**
   * Returns application type
   **/  

  public int getAT() {
       
    return AT;

  }  

 /**
   * Returns session number
   **/
  
  public int getSID() {
   
    return SID;
  }

 /**
   * Returns local user number
   **/

  public int getLocalUID() {
   
    return localUID;
  }
  
    public String toString()
    {
      return CL+"[AT="+AT+",SID="+SID+",UID="+localUID+"]";
    }// toString __________________________________________________________


}// RaskMessage ///////////////////////////////////////////////////////////


