// $Id: SupdMessage.java,v 1.4 1998/07/10 20:15:39 beca Exp $
/*
File: SupdMessage.java
Component: Server, Plugin
Date		Author		Changes
Aug 22 96	Lukasz Beca	Created
*/


package webwisdom.tango.messages;


import webwisdom.tango.messages.Message;
import webwisdom.tango.consts.Const;
import java.io.*;   


/**
 * A class representing SUPD message
 * 
 *@author Lukasz Beca
 *@see Central Server - Local Demon protocol
 **/
public class SupdMessage extends Message
{
    private static final String CL="SupdMessage";


    private int PARAM;
    private int SID;
    private int AT;
    private int UID;
    private String name;


/**
   * Creates a new SUPD message
   *@param PARAMparam update type: Const.ADDUPDATE, Const.REMOVEUPDATE, Const.MASTERUPDATE, Const.LOGINUPDATE, Const.LOGOUTUPDATE
   *@param SIDparam session number
   *@param ATpar application type
   *@param UIDparam user number
   *@param namepar user name
   **/

  public SupdMessage(int PARAMparam, int SIDparam, int ATpar, int UIDparam, String namepar) {

   PARAM = PARAMparam;
   UID = UIDparam;
   AT = ATpar;
   SID = SIDparam;
   name = new String(namepar);
   type = Const.SUPD;
  }

 /**
   * Creates a new SUPD message, information is read from the input stream
   *@param instream  input stream
   **/

  SupdMessage(DataInputStream instream) throws IOException {
       
    PARAM = instream.readInt();     
    SID = instream.readInt();
    AT = instream.readInt();
    UID = instream.readInt();
    name = instream.readUTF();

    type = Const.SUPD;
    
  }

 /**
   * Writes SUPD message to the output stream
   *@param outstream  output stream
   **/


  void send(DataOutputStream outstream) throws IOException {
  
    
 //   System.out.println("Sending type " + Const.SUPD);
    outstream.writeInt(Const.SUPD);
 //   System.out.println("Sending PARAM " + PARAM);
    outstream.writeInt(PARAM);
 //   System.out.println("Sending SID " + SID);
    outstream.writeInt(SID);
 //   System.out.println("Sending AT " + AT);
    outstream.writeInt(AT);
 //   System.out.println("Sending UID " + UID);
    outstream.writeInt(UID);
 //   System.out.println("Sending name " + name);
    outstream.writeUTF(name);

    outstream.flush();
   
  }
  

 /**
   * Returns user number
   **/  


  public int getUID() {
       
    return UID;

  }  

 /**
   * Returns session number
   **/        

  public int getSID() {
   
    return SID;
  }

 /**
   * Returns update type
   **/  

  public int getPARAM() {
   
    return PARAM;
  }

 /**
   * Returns application type
   **/  

   public int getAT() {
   
    return AT;

  }

 /**
   * Returns user name
   **/  

   public String getName() {
   
    return new String(name);

  }

  
    public String toString()
    {
      return CL+"[AT="+AT+",SID="+SID+",UID="+UID+",PARAM="+PARAM+"]";
    }// toString __________________________________________________________


}// SupdMessage ///////////////////////////////////////////////////////////


