
package webwisdom.tango.object;
import java.io.*;

/**
  * A class representing messages sent among TObjects. This object of this
  * class is supposed to be created only by TNode object. Then the user
  * may write to the MessageOutputStrem object as if it was an ordinary
  * DataOutputStream object. When the method <B>send()</B> is called the
  * written data is send to other applications in the session.
  *
  * For example:
  * <pre>
  *	TNode tnode;
  *	MessageOutputStrem mos=tnode.getMOS((TObject)to,false);
  *	mos.writeInt(124);
  *	mos.writeUTF("abcd");
  *	mos.send();
  * </pre>
  *
  * @author Piotr Sokolowski
  * @version $Revision: 1.5 $
  * @see webwisdom.tango.object.TObject
  * @see webwisdom.tango.object.TNode
  */
public class MessageOutputStream extends DataOutputStream
{
  private ByteArrayOutputStream bos;
  private TNode tn;
  private boolean isRemovable;
  /**
    * Creates a new MessageOutputStrem object.
    * @param tnode the reference to TNode which creates the MessageOutpusStream object.
    * @param removable defines if the message may be discarded in case of traffic congestion.
    * @param bos defines an empty ByteArrayOutputStream object which will store data written to the MessageOutputStream.
    * @see webwisdom.tango.object.TNode
    */
  MessageOutputStream(TNode tn,boolean removable,ByteArrayOutputStream bos)
    {
      super(bos);
      this.bos=bos;
      this.tn=tn;
      isRemovable=removable;
    }

  /**
    * Method used by TNode to get the ByteArrayOutputStream during broadcasting the message.
    * @return Returns a reference to ByteArrayOutputStream where all data written to the MessageOutputStream resides.
    */
  ByteArrayOutputStream getByteStream()
    {
      return bos;
    }
  /**
    * Used to initiate broadcasting of the Message. The broadcasting itself is performed by the TNode object.
    * @see webwisdom.tango.object.TNode
    */
  public void send()
    {
      tn.sendData(this,isRemovable);
    }
}
