// $Id: ServerProtocol.java,v 1.20 1999/04/22 23:21:35 beca Exp $
/*
File: ServerProtocol.java
Component: Server
Date		Author		Changes
Aug 24 96	Lukasz Beca	Created
Aug 26 96       Lukasz Beca     Messages JASK and JANS added
Nov 15 96       Lukasz Beca     Bug in handle*ANS eliminated
Dec 02 96       LUkasz Beca     wake() eliminated from all functions
*/


package webwisdom.tango.protocols;


import java.net.*;
import java.util.*;
import webwisdom.tango.protocols.Protocol;
import webwisdom.tango.messages.*;
import webwisdom.tango.threads.*;
import webwisdom.tango.consts.Const;
import webwisdom.tango.structures.*;
import webwisdom.tango.test.Log;
import webwisdom.tango.server.LicenseManager;
import webwisdom.tango.server.License;


public class ServerProtocol extends Protocol
{
    private static final String CL="ServerProtocol";


    protected ThreadOut out;
    protected ComThread c;
    protected DbThread d;
    protected Timeout _t;  

    protected int UID;

    private boolean logged_in;


    public ServerProtocol(ThreadOut outparam,ComThread cparam,
			  DbThread dparam, Timeout t)
    {
      out = outparam;
      c = cparam;
      d = dparam;
      _t = t;
      UID = 0;
      logged_in = false;
    }	


    public void handleJOIN(JoinMessage m)
    {
      Log.out.println(CL+"."+UID+".handleJOIN("+m+")", 2);

      int SID = m.getSID();
      int AT = m.getAT();
      int uniqueAID; 
      CommandMessage comm;
      ThreadOut tmp;
      int masterUID;
      int newSID;
      JaskMessage jask;
      
      
      if(!logged_in)
      {
	Log.err.println(CL+"."+UID+".handleJOIN(): user not logged!", 2);
	return;
      }


      //if (AI in SID for UID(UID)) stop
      synchronized(d)
      {    // start of operations on system state data
        if(!d.getApplications().inSession(SID, UID))
	{
	  if(SID == 0)
	  {
	    // SID  = generate new SID    
	    newSID = assignSID();
	    //INSERT INTO SESSIONS (SID, AT, MASTER)
	    d.getSessions().addSession(newSID, AT, UID);
	    c.getSessionTable().addSession(newSID);
	    Log.out.println(CL+"."+UID+".handleJOIN(): new SID="+newSID+", type="+d.getSessions().getType(newSID), 2);
	    //INSERT INTO APPLICATIONS (AID, SID, UID(UID))
	    //generate unique AID
	    uniqueAID = assignAID();
	    d.getApplications().addApplication(uniqueAID, newSID, UID);
	    //   c.getSessionTable().addApplication(newSID, out, uniqueAID);
	    Log.out.println(CL+"."+UID+".handleJOIN(): added AID="+uniqueAID+" to SID="+newSID+" and disabled", 2);
	    //SEND COMMAND(AT, params, AID, newSID)
	    comm = new CommandMessage(AT, "EMPTY", uniqueAID, newSID);
            out.putMessage(comm);  
	          
	  }
	  else
	  {
	    //SELECT mUID = master FROM SESSIONS WHERE SID = SID
	    // SEND_TO (mUID, JASK(UID, SID))
	    masterUID = d.getSessions().getMaster(SID);
	    if(masterUID == -1)
	    {
	      Log.err.println(CL+"."+UID+".handleJOIN(): SID="+SID+" does not exist!", 2);
	      return;
	    }
	    tmp = c.getThread(masterUID);
	    jask = new JaskMessage(AT, SID, UID);
	    tmp.putMessage(jask);
	  } 
	}
	else
	{
	  Log.err.println(CL+"."+UID+".handleJOIN(): user already in session!", 2);
	}
      }  //end of operations on system state data
      d.getApplications().list();
      d.getSessions().list();
    }// handleJOIN _________________________________________________________


    public void handleJANS(JansMessage m)
    {
      Log.out.println(CL+"."+UID+".handleJANS("+m+")", 2);

      int AT = m.getAT();
      int SID = m.getSID();
      int lUID = m.getUID();
      int answer = m.getAnswer();
      int uniqueAID;
      ThreadOut tmp;
      //SupdMessage update;
      CommandMessage comm;
      //String name;

      if(!logged_in)
      {
	Log.err.println(CL+"."+UID+".handleJANS(): user not logged!", 2);
	return;
      }

      synchronized(d)
      {   //start of operations on system state data
	if (!d.getParticipants().isLoggedIn(lUID))
	{
	  Log.err.println(CL+"."+UID+".handleJANS(): UID="+lUID+" not logged in!", 2);
	  return;
	}

	//IF "no" stop
	if (answer == Const.NO)
	{
	  Log.out.println(CL+"."+UID+".handleJANS(): master does not agree", 2);
	  Log.out.printSESSION_JOIN(d.getUsers().getName(lUID), lUID, d.getUsers().getName(UID), UID, AT, SID, 0, "NI", Log.FAILED_DENIED, new Date());
          tmp = c.getThread(lUID); 
	  if (tmp != null)
	  {
	    tmp.putMessage(m);
	  }
	  else
	  {
	    Log.err.println(CL+"."+UID+".handleJANS(): UID="+lUID+" not logged in", 2);
	  } 
	  return;
	}

  
	if (d.getSessions().getMaster(SID) == -1)
	{
	  Log.err.println(CL+"."+UID+".handleJANS(): SID="+SID+" does not exist!", 2);
	  return;
	}


	if (d.getApplications().inSession(SID, lUID))
	{
	  Log.err.println(CL+"."+UID+".handleJANS(): UID="+lUID+" already in SID="+SID, 2);
	  return;
	}

   
	if (AT != d.getSessions().getType(SID))
	{ // check whether type is correct
	  Log.err.println(CL+"."+UID+".handleJANS(): wrong AT!", 2);
	  return;
	}


	//generate unique AID
	uniqueAID = assignAID();
	tmp = c.getThread(lUID);
	//INSERT INTO APPLICATIONS(AID, SID, lUID)
	d.getApplications().addApplication(uniqueAID, SID, lUID);
	//c.getSessionTable().addApplication(SID, tmp, uniqueAID);
	Log.out.println(CL+"."+UID+".handleJANS(): AID"+uniqueAID+" added to SID="+SID+" and disabled", 2);
	//SEND_TO(UID, COMMAND(AT, params, AID, SID))
	comm = new CommandMessage(AT, "EMPTY", uniqueAID, SID);
	tmp.putMessage(comm);
      } //end of operations on system state data   
    }// handleJANS _________________________________________________________



    public void handleSTARTED(StartedMessage m)
    {
      Log.out.println(CL+"."+UID+".handleSTARTED("+m+")", 2);

      int AID = m.getAID();
      int messageSID = m.getSID();
      int SID;
      SupdMessage update, mupdate;
      int type;
      String name;

      if(!logged_in)
      {
	Log.err.println(CL+"."+UID+".handleSTARTED(): user not logged!", 2);
	return;
      }
    
    
      synchronized(d)
      {   //start of operations on system state data
        SID = d.getApplications().getSession(AID);
	if (SID == -1)
	{
	  Log.err.println(CL+"."+UID+".handleSTARTED(): AID="+AID+" does not exists!", 2);
	  update = new SupdMessage(Const.REMOVEUPDATE, messageSID, 0, 0, "");  //AT = 0;
	  out.putMessage(update);
	  return;
	}
	d.getApplications().enableApp(AID);
        if (!d.getSessions().isEnabled(SID))
	{
          d.getSessions().enableSession(SID);
        }
	c.getSessionTable().addApplication(SID, out, AID);
	Log.out.println(CL+"."+UID+".handleSTARTED(): AID="+AID+" enabled", 2);
	name = d.getUsers().getName(UID);
	type = d.getSessions().getType(SID);
	if (d.getSessions().getMaster(SID) == UID)
	{
	  Log.out.println(CL+"."+UID+".handleSTARTED(): AID="+AID+" is master", 2);
	  update = new SupdMessage(Const.ADDUPDATE, SID, type, UID, name);
	  mupdate =  new SupdMessage(Const.MASTERUPDATE, SID, type, UID, name);
	  //SEND_TO_ALL SUPD(add, SID, AT, UID(UID), username)
	  sendToAll(update);
	  // SEND_TO_ALL SUPD(master, SID, 0, UID(UID), username);
	  sendToAll(mupdate);
	  Log.out.printSESSION_START(name, UID, type, SID, Log.OK, new Date());
	}
	else
	{
	  Log.out.println(CL+"."+UID+".handleSTARTED(): AID="+AID+" is not master", 2);
	  update = new SupdMessage(Const.ADDUPDATE, SID, type, UID, name);    
	  //SEND_TO_ALL SUPD(add, SID, UID(UID))
	  sendToAll(update);
	  
	  int MUID = d.getSessions().getMaster(SID);
	  Log.out.printSESSION_JOIN(name, UID, d.getUsers().getName(MUID), MUID, type, SID, AID, "NI", Log.OK, new Date());
	}
      } // end of operations on system state data
    }// handleSTARTED ______________________________________________________


    public void handleFINISH(FinishMessage m)
    {
      Log.out.println(CL+"."+UID+".handleFINISH("+m+")", 2);

      int AID = m.getAID();
      int user;

      //SELECT s = SID FROM APPLICATIONS WHERE aid = AID
      //DELETE FROM APPLICATIONS WHERE aid = AID
      //SELECT M = S.master FROM SESSIONS WHERE aid = AID
      //IF (M == UID(UID)) DELETE FROM APPLICATIONS WHERE SID = s
      
      if(!logged_in)
      {
	Log.err.println(CL+"."+UID+".handleFINISH(): user not logged!", 2);
	return;
      }
  
      synchronized(d)
      {   //start of operations on system state data
	user = d.getApplications().getUser(AID);
	if (user != UID)
	{
	  if (user == -1) {
	    Log.err.println(CL+"."+UID+".handleFINISH(): AID="+AID+" does not exist!", 2);
//	    Log.out.printSESSION_TERM(d.getUsers().getName(UID), UID, 0, 0, Log.FAILED, new Date());
	    
	  } else {
	    Log.err.println(CL+"."+UID+".handleFINISH(): wrong UID!", 2);
	    
//	    int SID = d.getApplications().getSession(AID);
	    
//	    Log.out.printSESSION_TERM(d.getUsers().getName(UID), UID, d.getSessions().getType(SID), SID, Log.FAILED, new Date());
	    
	  }
	  return;
	  
	  
	}
	finish(AID);
      } //end of operations on system state data            
      d.getApplications().list();
      d.getSessions().list();   
    }// handleFINISH _______________________________________________________




    public void handleEVENT(EventMessage e)
    {
      //System.out.println(CL+"."+UID+".handleEVENT("+e+")");
      
      if(!logged_in)
      {
	Log.err.println(CL+"."+UID+".handleEVENT(): user not logged!", 2);
	return;
      }
 
      int AID=e.getAID();

      // *******************  tests only ******************
//    int len = e.getLen();
//    byte data = e.getData();
//   System.out.println("T " + UID  + " ServerProtocol: EVENT message accepted");
//    System.out.println("T " + UID  + " ServerProtocol: Data AID" + AID + " length " + len);
//    System.out.println("T " + UID  + " ServerProtocol: Data :");
      
//    for(int i = 0; i < len; i ++) {
      
//       System.out.print(data[i]);
//    }
//    System.out.println("");
  
      // ******************  tests only **********************

      if(AID==Const.CRLAID)
      {
	sendControl(e);
      }
      else
      {
	c.putMessage(e);
      }
    }// handleEVENT _________________________________________________________


    public void handleRJOIN(RjoinMessage m)
    {
      Log.out.println(CL+"."+UID+".handleRJOIN("+m+")", 2);

      int AT = m.getAT();
      int SID = m.getSID();
      int rUID = m.getRemoteUID();

      if(!logged_in)
      {
	Log.err.println(CL+"."+UID+".handleRJOIN(): user not logged!", 2);
	return;
      }

      synchronized(d)
      { //start of operations on system state data
	if (d.getSessions().getMaster(SID) == -1)
	{
	  Log.err.println(CL+"."+UID+".handleRJOIN(): SID="+SID+" does not exist!", 2);
	  return;
	}
	int masterUID = d.getSessions().getMaster(SID);
	if (masterUID != UID)
	{
	  Log.err.println(CL+"."+UID+".handleRJOIN(): RJOIN message send by non-master user!", 2);
	  return;
	}
	if (!d.getParticipants().isLoggedIn(rUID))
	{
	  Log.err.println(CL+"."+UID+".handleRJOIN(): rUID="+rUID+" not logged in!", 2);
	  return;
	}
	//SELECT C = count(*) FROM APPLICATIONS WHERE SID = SID AND RUID = UID
	// IF (C > 0) stop
	if (d.getApplications().inSession(SID, rUID))
	{
	  Log.out.println(CL+"."+UID+".handleRJOIN(): rUID="+rUID+" already in SID="+SID, 2);
	  return;
	}
	if (AT != d.getSessions().getType(SID))
	{ // check whether type is correct
	  Log.err.println(CL+"."+UID+".handleRJOIN(): wrong AT!", 2);
	  return;
	}
	//SEND_TO (RUID, RASK(AT, SID, UID(UID)))
	RaskMessage rask = new RaskMessage(AT, SID, UID);
	ThreadOut rout = c.getThread(rUID);
	rout.putMessage(rask);
      } //end of operations on system state data
    }// handleRJOIN ________________________________________________________

  
    public void handleRANS(RansMessage m)
    {
      Log.out.println(CL+"."+UID+".handleRANS("+m+")", 2);

      int AT = m.getAT();
      int SID = m.getSID();  
      int answer = m.getAnswer();
      int rUID = m.getRemoteUID();
      int lUID = m.getLocalUID();


      if(!logged_in)
      {
	Log.err.println(CL+"."+UID+".handleRANS(): user not logged!", 2);
	return;
      }

      synchronized(d)
      {  //start of operations on system state data
	if (d.getSessions().getMaster(SID) == -1) {
	  Log.err.println(CL+"."+UID+".handleRANS(): SID="+SID+" does not exist!", 2);
	  return;
	}
	if (!d.getParticipants().isLoggedIn(lUID))
	{
	  Log.err.println(CL+"."+UID+".handleRANS(): UID="+lUID+" not logged in!", 2);
	  return;
	}
	int masterUID = d.getSessions().getMaster(SID);
	if (masterUID != lUID)
	{
	  Log.err.println(CL+"."+UID+".handleRANS(): RANS message send by non-master user!", 2);
	  return;
	}
	if (rUID != UID)
	{
	  Log.err.println(CL+"."+UID+".handleRANS(): rUID!=UID", 2);
	  return;
	}

	//IF "no" stop
	if (answer == Const.NO)
	{
	  Log.out.println(CL+"."+UID+".handleRANS(): user does not agree for joining session", 2);
	  Log.out.printSESSION_JOIN(d.getUsers().getName(rUID), rUID, d.getUsers().getName(lUID), lUID, AT, SID, 0, "NI", Log.FAILED_REMOTEDENIED, new Date());
	  ThreadOut tmp = c.getThread(lUID);
	  if (tmp != null)
	  {
	    tmp.putMessage(m);
	  }
	  else
	  {
	    Log.err.println(CL+"."+UID+".handleRANS(): UID="+lUID+" not logged in!", 2);
	  }
	  return;
	}
	
	if (d.getApplications().inSession(SID, UID))
	{
	  Log.err.println(CL+"."+UID+".handleRANS(): UID="+UID+" already in SID="+SID, 2);
	  return;
	}
	if (AT != d.getSessions().getType(SID))
	{ // check whether type is correct
	  Log.err.println(CL+"."+UID+".handleRANS(): wrong AT!", 2);
	  return;
	}

	//IF (yes) 
	//generate unique AID
	int uniqueAID = assignAID();
	//INSERT INTO APPLICATIONS (AID, SID, UID(UID))
        d.getApplications().addApplication(uniqueAID, SID, UID);
	//  c.getSessionTable().addApplication(SID, out, uniqueAID);
        Log.out.println(CL+"."+UID+".handleRANS(): AID="+uniqueAID+" added to SID="+SID, 2);
	//SEND COMMAND(AT, params, AID, SID)
	CommandMessage comm=new CommandMessage(AT,"EMPTY",uniqueAID,SID);
	out.putMessage(comm);
	//String name=d.getUsers().getName(UID);
	//SupdMessage update=new SupdMessage(Const.ADDUPDATE,SID,AT,UID,name);
	//SEND_TO_ALL SUPD(add, SID, AT, UID(UID), username)
	// sendToAll(update);
      } //end of operations on system state data
    }// handleRANS _________________________________________________________


    public void handleMASTER(MasterMessage m)
    {
      Log.out.println(CL+"."+UID+".handleMASTER("+m+")", 2);

      int SID = m.getSID();

      if(!logged_in)
      {
	Log.err.println(CL+"."+UID+".handleMASTER(): user not logged!", 2);
	return;
      }
   
      synchronized(d)
      {  //start of operations on system state data
	if (d.getSessions().getMaster(SID) == -1)
	{
	  Log.err.println(CL+"."+UID+".handleMASTER(): SID="+SID+" does not exist!", 2);
	  return;
	}
	//SELECT  M = MASTER FROM SESSIONS WHERE SID = SID
	int master = d.getSessions().getMaster(SID);
	// if (M == UID(UID) || M == null) stop;
	if (master == UID)
	{
	  Log.err.println(CL+"."+UID+".handleMASTER(): already master of SID="+SID+"!", 2);
	  return;
	}
	if (!d.getApplications().inSession(SID, UID))
	{
	  Log.err.println(CL+"."+UID+".handleMASTER(): not in SID="+SID+"!", 2);
	  return;
	}
	ThreadOut tout = c.getThread(master);
	//SEND_TO(M, MASK(SID, UID(UID))
	MaskMessage mask = new MaskMessage(SID, UID);
	tout.putMessage(mask);
      } //end of operations on system state data
    }// handleMASTER _______________________________________________________


    public void handleMANS(MansMessage m)
    {
      Log.out.println(CL+"."+UID+".handleMANS("+m+")", 2);

      SupdMessage update;
      int answer = m.getAnswer(); 
      int SID = m.getSID();
      int lUID = m.getUID();
      Vector outVector;
      ThreadOut tmp;
      int i;
      String name;
      int master;
      int AT;

      if(!logged_in)
      {
	Log.err.println(CL+"."+UID+".handleMANS(): user not logged!", 2);
	return;
      }

      synchronized(d)
      {  //start of operations on system state data
	if (d.getSessions().getMaster(SID) == -1) {
	  Log.err.println(CL+"."+UID+".handleMANS(): SID="+SID+" does not exist!", 2);
	  return;
	}
	if (!d.getParticipants().isLoggedIn(lUID)) {

      Log.err.println(CL+"."+UID+".handleMANS(): UID="+lUID+" not logged in!", 2);
     return;
   }


   if (!d.getApplications().inSession(SID, lUID)) {

     Log.err.println(CL+"."+UID+".handleMANS(): UID="+lUID+" not in SID="+SID, 2);
     return;
   }




   master = d.getSessions().getMaster(SID);
   
   if (master != UID) {

     Log.err.println(CL+"."+UID+".handleMANS(): message send by non-master !", 2);
       
     return;

   }

 
   if (master == lUID) {
    
     Log.err.println(CL+"."+UID+".handleMANS(): UID="+lUID+" already master of SID="+SID, 2);
     return;

   }


   if (!d.getParticipants().isLoggedIn(lUID)) {

      Log.err.println(CL+"."+UID+".handleMANS(): UID="+lUID+" not logged in!", 2);
     return;
   }



   //IF "no" stop

   if (answer == Const.NO) {

     Log.out.println(CL+"."+UID+".handleMANS(): master does not agree for change of master", 2);
     tmp = c.getThread(lUID);
     if (tmp != null) {
       tmp.putMessage(m);
    
     }
     else {
      Log.out.println(CL+"."+UID+".handleMANS(): UID="+lUID+" not logged in!", 2);
     } 
       return;
   }

   
   
  

   
  // if (yes) 

    //UPDATE SESSIONS SET master = UID WHERE SID = SID
    
    d.getSessions().changeMaster(SID, lUID);

    // SEND_TO_ALL (SUPD(master, SID, UID))

    name = d.getUsers().getName(lUID);
    AT = d.getSessions().getType(SID);

    update = new SupdMessage(Const.MASTERUPDATE, SID, AT, lUID, name);
      
     
      //SEND_TO_ALL SUPD(master, SID, lUID)

    sendToAll(update); 
    
    
    }  // end of operations on system state data

    }// handleMANS _________________________________________________________


    public void handleLOGIN(LoginMessage m)
    {
      Log.out.println(CL+"."+UID+".handleLOGIN("+m+")", 2);

      LogaccMessage acc;
      SupdMessage update;
      String name;
      String passwd;
      String host, plainHost;
      String ip;
      String status, fullName, eMail;
      DbSessions dbs;
      Applications app;
      Vector prt;
      int size;
      int i;
      int masterUID;
      int SID;
      int AT;
      int usr;
      int check;
      Integer tmp;
      String tmpname = null;
      long time;
      GregorianCalendar expDate, now;
//      boolean pUsed;
      
      time = System.currentTimeMillis();

      name = m.getUsername();
      passwd = m.getPasswd();
      plainHost = n.getHost();
      host = plainHost + " " + (new Date(time)).toString();
      ip = n.getIP();


      if(logged_in)
      {
	Log.err.println(CL+"."+UID+".handleLOGIN(): user already logged!", 2);
	return;
      }
      
      
      if (LicenseManager.getInstance().getLicenseType() == License.EVALUATION) {
  
	      now = new GregorianCalendar();
		    expDate = LicenseManager.getInstance().getExpirationDate();
    
	      if (now.after(expDate)) {
    
		      Log.out.println(CL+"."+UID+".handleLOGIN(): license expired!", 2); 
			    acc = new LogaccMessage(name,0,"LICENSE_EXPIRED");
		      UID = 0;
		      out.putMessage(acc); 
		      Log.out.printUSER_LOGIN(name, UID, Log.NONE, plainHost, ip, Log.FAILED_LICENSE_EXPIRED, new Date());
		      return;    
	      }
  
     }
      
    
      synchronized(d)
      {  //start of operations on system state data
            
  //check licence constraints    
      
       if (LicenseManager.getInstance().getLicenseType() == License.COMMERCIAL) {
  
		     if (LicenseManager.getInstance().getAllowedUsers() <= d.getUsers().getUsersNumber()) {
	
		      Log.out.println(CL+"."+UID+".handleLOGIN(): license limit expired!", 2); 
			    acc = new LogaccMessage(name,0,"LIMIT_EXCEEDED");
		      UID = 0;
		      out.putMessage(acc); 
		      Log.out.printUSER_LOGIN(name, UID, Log.NONE, plainHost, ip, Log.FAILED_LIMIT_EXCEEDED, new Date());
		      return;
    
		     } 
	    }          
	  	          
      
	//CHECK IF USER IS ALREADY LOGGED IN
	check = d.getUsers().getUID(name);
	if (check != -1)
	{
	  Log.out.println(CL+"."+UID+".handleLOGIN(): user already logged!", 2); 
	  acc = new LogaccMessage(name,0,"USER_PRESENT");
	  UID = 0;
	  out.putMessage(acc); 
	  Log.out.printUSER_LOGIN(name, UID, Log.NONE, plainHost, ip, Log.FAILED_PRESENT, new Date());
	  return;
	}   
	
	if ( d.getUsers().passwdUsed() && !d.getUsers().checkPasswd(name, passwd)) {
	
	  Log.out.println(CL+".handleLOGIN(): user " + name + " tried to log in with invalid password", 2); 
	  acc = new LogaccMessage(name,0,"INVALID_PASSWORD");
	  UID = 0;
	  out.putMessage(acc); 
	  Log.out.printUSER_LOGIN(name, UID, Log.NONE, plainHost, ip, Log.FAILED_PASSWORD, new Date());
	  return;	   
	    
	}  	    
	        
	// ADD new user TO USER structures
	d.getUsers().addUser(name, " ", " ");
	UID = d.getUsers().getUID(name);
	//SEND LOGACC with privileges = ACCESS
	logged_in = true;
	c.addThread(out, UID);  // adding threadout to table in commthread
	acc = new LogaccMessage(name, UID, d.getUsers().getPriv(UID));

	    
	out.putMessage(acc);
	Log.out.println(CL+"."+UID+".handleLOGIN(): user logged in with UID="+UID, 2);
	Log.out.printUSER_LOGIN(name, UID, d.getUsers().getStatus(name), plainHost, ip, Log.OK, new Date());
	//INSERT INTO PARTICIPANTS
	d.getParticipants().addParticipant(UID, host, ip);
	app = d.getApplications();
	dbs = d.getSessions();
	prt = d.getParticipants().getUsers();
	_t.newUser(UID, this);  //add to the timeout structures
	//SEND INFORMATION ABOUT LOGGED IN USERS
	size = prt.size();
	for(i = 0; i < size; i ++)
	{
	  tmp = (Integer) prt.elementAt(i);
	  usr = tmp.intValue();
	  tmpname = d.getUsers().getName(usr);
	  status = d.getUsers().getStatus(tmpname);
	  if (status == null) status = "";
	  fullName = d.getUsers().getFullName(tmpname);
	  if (fullName == null) fullName = "";
	  eMail = d.getUsers().getEMail(tmpname);
   	if (eMail == null) eMail = "";
	  name = d.getUsers().getName(usr) + "#" + d.getParticipants().getHost(usr) + "#" + d.getParticipants().getIP(usr) + "#" + status + "#" + fullName + "#" + eMail;
	  if (usr != UID)
	  {
	    update = new SupdMessage(Const.LOGINUPDATE, 0, 0, usr, name);
	    out.putMessage(update);
	  }
	}

	// SEND_TO_ALL INFORMATION ABOUT NEW USER LOGGED IN
	tmpname = d.getUsers().getName(UID);
	status = d.getUsers().getStatus(tmpname);
	if (status == null) status = "";
	fullName = d.getUsers().getFullName(tmpname);
	if (fullName == null) fullName = "";
	eMail = d.getUsers().getEMail(tmpname);
	if (eMail == null) eMail = "";
	name = d.getUsers().getName(UID) + "#" + d.getParticipants().getHost(UID) + "#" + d.getParticipants().getIP(UID) + "#" + status + "#" + fullName + "#" + eMail;
	update =  new SupdMessage(Const.LOGINUPDATE, 0, 0, UID, name);
	sendToAll(update);
	// SEND INFORMATION ABOUT SESSIONS
	size = app.getSize();
	for(i = 0; i < size; i ++)
	{
	  if (app.isEnabledIndex(i))
	  {
	    SID = app.getSID(i);
	    usr = app.getUID(i);
	    AT = dbs.getType(SID);
	    tmpname = d.getUsers().getName(usr);
	    update = new SupdMessage(Const.ADDUPDATE, SID, AT, usr, tmpname);
	    out.putMessage(update);
	  }
	}
	size = dbs.getSize();
	//SEND INFORMATION ABOUT MASTERS
	for(i = 0; i < size; i ++)
	{
	  masterUID = dbs.getMasterUID(i);
	  if(masterUID != 0 && dbs.isEnabledIndex(i))
	  {
	    SID = dbs.getSID(i);
	    AT = dbs.getAT(i);
	    tmpname = d.getUsers().getName(masterUID);
	    update = new SupdMessage(Const.MASTERUPDATE, SID, AT, masterUID, tmpname);
	    out.putMessage(update);
	  }
	}
      } //end of operations on system state data
      d.getParticipants().list();
    }// handleLOGIN ________________________________________________________
 

    protected void handleLOGOUT(LogoutMessage m)
    {
      Log.out.println(CL+"."+UID+".handleLOGOUT("+m+")", 2);

      int UIDpar=m.getUID();
  
      if(!logged_in)
      {
	Log.err.println(CL+"."+UID+".handleLOGOUT(): user not logged!", 2);
	return;
      }

      if(UIDpar!=UID)
	Log.err.println(CL+".handleLOGOUT(): wrong UID!", 2);

      synchronized(d)
      { //start of operations on system state data
	logout(Const.OK);
      } //end of operations on system state data
      
      d.getParticipants().list();
    }// handleLOGOUT _______________________________________________________


    public void handlePING(PingMessage a)
    {
      //System.out.println(CL+"."+UID+".handlePING("+a+")");
      _t.userResponded(UID);
    }// handlePING _________________________________________________________
		
	
    public void handleGROUP_EVENT(GroupEventMessage m)
    {
    //  System.out.println(CL+"."+UID+".handleGROUP_EVENT("+m+")");
	
      byte[] data = m.getData();
      int AID = m.getAID();
      int[] userlist=m.getUserList();
           
      if (AID == Const.CRLAID) {
	
		    for(int i = 0; i < userlist.length; i ++) {
		      int groupUID = userlist[i];		
			    ThreadOut tmp = c.getThread(groupUID);		 
		      EventMessage em = new EventMessage(Const.CRLAID, data);
		      if (tmp != null) {
			      tmp.putMessage(em);
		      }
		    }		            
      
      } else {
      
          int SID = d.getApplications().getSession(AID);
	        for(int i = 0; i < userlist.length; i ++) {
			      int groupUID = userlist[i];		      		
			      int newAID = d.getApplications().getAID(SID, groupUID);
			      ThreadOut tmp = c.getThread(groupUID);		 
			      EventMessage em = new EventMessage(newAID, data);
			      if (tmp != null) {
				      tmp.putMessage(em);
			      }		      
				  }		  
      }            
    }
    // handleGROUP_EVENT __________________________________________________
    
    
    public void finish(int AIDpar)
    {
      int SID = d.getApplications().getSession(AIDpar);  
      int userUID = d.getApplications().getUser(AIDpar);
      int masterUID = d.getSessions().getMaster(SID);
      boolean sessionIsEnabled = d.getSessions().isEnabled(SID);
      boolean appIsEnabled = d.getApplications().isEnabled(AIDpar);
      boolean closed = d.getApplications().removeApplication(AIDpar);
      //IF (!SELECT count(*) FROM APPLICATIONS WHERE SID = s
      //DELETE FROM SESSIONS WHERE SID = s
      masterUID = d.getSessions().getMaster(SID);
      int AT = d.getSessions().getType(SID); // added
      SupdMessage update;
      if (closed || (masterUID == userUID))
      {
	      Log.out.println(CL+".finish(): Session " + SID + " closed", 2);
	      d.getApplications().removeFromSession(SID);   
	      d.getSessions().removeSession(SID);
	      
	      if (sessionIsEnabled) {
	      
		      c.getSessionTable().removeSession(SID);
	//SEND_TO_ALL SUPD(remove, SID, 0)		     	
		      update = new SupdMessage(Const.REMOVEUPDATE, SID, AT, 0, "");
		      Log.out.printSESSION_TERM(d.getUsers().getName(UID), UID, AT, SID, Log.OK, new Date());
		      sendToAll(update);
		    }

		 } else {
		 
		    Log.out.println(CL+".finish(): Application "+AIDpar+" deleted", 2);
	      //SEND_TO_ALL SUPD(remove, SID, UID(UID))	      	      	    
	      
	      if (appIsEnabled) {
	      
		      c.getSessionTable().removeApplication(AIDpar);
		      update = new SupdMessage(Const.REMOVEUPDATE, SID, AT, UID, "");
		      Log.out.printSESSION_LEAVE(d.getUsers().getName(UID), UID, d.getUsers().getName(masterUID), masterUID, AT, SID, AIDpar, "NI", Log.OK, new Date());
		    }
      }
          }// finish _____________________________________________________________


    public void logout(int status)
    {
      // SELECT applications FROM APPLICATIONS WHERE uid = UID
      Vector apps=d.getApplications().getApplications(UID);
      c.removeThread(UID); //from now information will not be sent to logged out user
      // FOR ALL applications FINISH
      for(int i = 0; i < apps.size(); i ++)
      {
	Integer tmp = (Integer) apps.elementAt(i);
 	finish(tmp.intValue());
      }
      //DELETE FROM PARTICIPANTS WHERE uid = UID
      _t.userRemoved(UID);  // remove from timeout structure
      d.getParticipants().deleteParticipant(UID);
      logged_in=false;
      String name=d.getUsers().getName(UID);
      // SEND_TO_ALL(INFORMATION ABOUT LOGGED OUT USER
      SupdMessage update=new SupdMessage(Const.LOGOUTUPDATE,0,0,UID,name);
      sendToAll(update);
      
      switch(status) {
      
	      case Const.OK:
      
		      Log.out.printUSER_LOGOUT(d.getUsers().getName(UID), UID, Log.OK, new Date());
		      break;
	      
	      case Const.CONNBROKEN:
	    
		      Log.out.printUSER_LOGOUT(d.getUsers().getName(UID), UID, Log.CONN_BROKEN, new Date());
		      break;
		      
		    case Const.TIMEOUT:
		    
		      Log.out.printUSER_LOGOUT(d.getUsers().getName(UID), UID, Log.TIMEOUT, new Date());
		      break;  		      
	    
	    }
      d.getUsers().removeUser(UID);
      
    //  UID=0;
    }// logout _____________________________________________________________


    private void sendToAll(Message m)
    {
    /*  Vector outVector=c.getAllThreads();
      for(int i=0;i<outVector.size();i++)
      {
	ThreadOut tmp=(ThreadOut)outVector.elementAt(i);
	tmp.putMessage(m);
      }*/
      
      Enumeration outThreads = c.getAllThreads();
      ThreadOut tmp;
      while(outThreads.hasMoreElements()) {
      
	      tmp = (ThreadOut) outThreads.nextElement();
	      tmp.putMessage(m);
	    }            
    }// sendToAll __________________________________________________________


    private void sendControl(Message m)
    {
     // Vector outVector=c.getAllThreads();
     
      Enumeration outThreads = c.getAllThreads();
      ThreadOut tmp;
   /*   int index=outVector.indexOf(out);
      if(index==-1)
      {
	System.out.println(CL+".sendControl(): cannot find sender thread");
      }
      else
      { 
	for(int i=0;i<outVector.size();i++)
	{
	  if(i!=index)
	  {
	    ThreadOut tmp=(ThreadOut)outVector.elementAt(i);
	    tmp.putMessage(m);
	  }
	}
      } */
      
      while(outThreads.hasMoreElements()) {
      
	      tmp = (ThreadOut) outThreads.nextElement();
	      if (tmp != out) {
	        Log.out.println(CL+".sendControl(): sending update", 2);
		      tmp.putMessage(m);
	      
	      }            
      }
      
      
    }// sendControl _______________________________________________________


    public void stop()
    {
      
    }// stop _______________________________________________________________


    private static int AIDno=1;
    private static int SIDno=1;


    private static synchronized int assignAID()
    {
      int r=AIDno;
      AIDno++;
      return r;
    }// assignAID _________________________________________________________


    private static synchronized int assignSID()
    {
      int r=SIDno;
      SIDno++;
      return r;
    }// assignSID __________________________________________________________


}// ServerProtocol /////////////////////////////////////////////////////////


