// $Id: PasswdManager.java,v 1.10 1999/03/09 20:46:40 beca Exp $
/*
File: PasswdManager.java
Component: Server
Date		Author		Changes
Nov 30 98	Lukasz Beca	Created
*/

package webwisdom.tango.server;

 
import java.io.*;
import java.util.*;
import java.security.*;
import webwisdom.tango.test.Log;


/*

PasswdManager class is used to retrieve and write data to server
configuration file. The file has following format:
1. Information about each user is stored in file as one line
2. Line has following structure:
name:password:status:fullname:e-mail
3. All lines that start with # character are treated as comments and
not interpreted by this class

Example:
#comment
luk:4.28.93.-76.-112.-99.26.23.-91.-101.41.32.-77.-49.-22.0.-30.-71.-28.-70:student:Witek Prytek:aaa@bbb.ccc.ddd
#a;dlkfjaldfks
tadeusz:11.24.-68.26.-52.-72.-111.57.-87.-87.52.-83.30.63.-2.-8.-91.-39.76.-49:prof:Wincenty Zielinski:ddd@aaa.hhh.rrr
#wacek i placek
#al;sdkjfsaldkjfaslkdf
placek:110.-70.117.-85.-89.57.4.-63.19.-40.-74.-29.-2.115.-98.-102.77.-34.-26.104:operator:Jozef Tkaczuk:ppp@iii.kkk.kkk
#ha

*/


public class PasswdManager {
  
  private String passwdFilename;
  private MessageDigest md;
  

  // constant name refers to the information stored on assigned position  
	    
  public static final int NAME = 1;
  public static final int PASSWORD = 2;
  public static final int STATUS = 3;
  public static final int FULLNAME = 4;
  public static final int EMAIL = 5;   
  public static final int ENUMBER = 5;  
  
  
  // constructor: Initializes class with provided filename.
  
  public PasswdManager(String filename) throws IOException {

      passwdFilename = filename;
      
      try { 
       
	      md = MessageDigest.getInstance("SHA");      	       	    
	      
	    } catch(Exception e) {
	    
	      e.printStackTrace(Log.err);
	    
	    } 	    	           
  }

  //close() Ends session with PasswdManager class.
      
  public void close() {
   
    passwdFilename = null;
    md = null;
      	
	}  			
  
  /*getPassword(String name ) Retrieves password of the 'name' user. If no password is
assigned, string of length 0 is returned (not null). If the
user 'name' is not present in the file or entry
format is invalid, null is returned. */
  
  public String getPassword(String name) {
  
      String encPasswd;
            
      encPasswd = get(name, PASSWORD);
                  
      return encPasswd;
  
  }
  
  /*  public String getStatus(String name) 

Retrieves status of the 'name' user. If no status is
assigned, string of length 0 is returned (not null). If the
user 'name' is not present in the file or entry
format is invalid, null is returned.
  */
  
  public String getStatus(String name) {
  
      String status;
            
      status = get(name, STATUS);
                  
      return status;
     
  }
  
  // following methods behave in similar fashion as  getStatus(String name)
  
  public String getFullName(String name) {
  
      String fullName;
            
      fullName = get(name, FULLNAME);
                  
      return fullName;
     
  }
  
  public String getEMail(String name) {
  
      String eMail;
            
      eMail = get(name, EMAIL);
                  
      return eMail;
     
  }
  
  /*
  public Hashtable getPasswordEntries()
  
  Returns Hashtable with all valid password entries in the 
file. Each entry is of the form: (name, encoded_password). 
Name is the key in the hash table. Method returns null if
the password file does not exist.
*/
  
  public Hashtable getPasswordEntries() {
  
    Hashtable result;
    
    result = getEntries(PASSWORD);
      
    return result;

  }

// following methods behave in similar fashion as getPasswordEntries() 
 
  public Hashtable getStatusEntries() {
  
    Hashtable result;
    
    result = getEntries(STATUS);
      
    return result;

  }
  
  public Hashtable getFullNameEntries() {
  
    Hashtable result;
    
    result = getEntries(FULLNAME);
      
    return result;

  }


  public Hashtable getEMailEntries() {
  
    Hashtable result;
    
    result = getEntries(EMAIL);
      
    return result;

  }
  
  
  /*  
  
  public boolean passwordOK(String name, String decPasswd) 
  
     Checks if the password is valid for 'name' user. Password
should be provided in decoded form e.g. "mlotek" or "slon",
not 
"-17.-75.-88.98.-80.101.-98.52.76.91.-111.14.120.69.-22.-68.15.104.59.125"
Returns 'true' if the password is valid, 'false' otherwise. 
    
 */   
  
  public boolean passwordOK(String name, String decPasswd) {
  
      String passwdFromFile;
      String encPasswd;            
      boolean result = false;	    
		        
      passwdFromFile = getPassword(name);
                           
      if (decPasswd == null || passwdFromFile == null) {
      
	       return false;
	
	    } 
           
      if (decPasswd.length() == 0 && passwdFromFile.length() == 0) {
      
	      result = true;
	      
      } else {
      
        encPasswd = getEncPasswd(decPasswd);
	      result = encPasswd.equals(passwdFromFile);
      
      }                   
      
      return result;      
    
  }
  
  /*
    
    public void addEntry(String name, String decPasswd, String status, String fullName, String eMail) throws IOException 
  
Writes user entry in the file. Entry has the format:

name:encoded_password:status

If the entry for 'name' user already exists in the file, the content 
of the entry is overwritten. 
If the entry for 'name' user does not exist, the new entry is added in the
file. 

Special case: it is possible to assign empty ("") password and
status to the user. 

*/
  
  
  public void addEntry(String name, String decPasswd, String status, String fullName, String eMail) throws IOException {
  
     String newName;   
     String oldName;
     String userName;
     File oldFile;
     File oldPasswdFile;
     FileWriter passwdFileWrite;
	   BufferedWriter bWriteFile;
	   String line, newLine;
	   int namePos, separatorPos;
	   boolean existed = false;
	   BufferedReader bufferedFile;
	   String encPasswd;
	   FileReader passwdFileRead;
	   File passwdFile;
	   
	
	   // check if all fields are valid
	         
     if (name == null || decPasswd == null || status == null || fullName == null || eMail == null) {
     
       return;
     
     }
	 
	         
		 // get encrypted version of password
		 
     encPasswd = getEncPasswd(decPasswd);          

	    // create a name of temporary file
		        
	   newName = passwdFilename;
     oldName = newName + ".tmp";
   
     // open temporary file
   
     oldPasswdFile = new File(oldName);
     
     // open current config file
     
     passwdFile = new File(passwdFilename);     
     
     //make current file temporary file
     
     passwdFile.renameTo(oldPasswdFile);     
    
     
     try {
     
       // create stream to read from the file
     
       passwdFileRead = new FileReader(oldPasswdFile);     
       bufferedFile = new BufferedReader(passwdFileRead);     
       
       // read line from the stream
       
       line = bufferedFile.readLine();
       
     } catch(FileNotFoundException e) {
          
       line = null;
       bufferedFile = null;
     }
	   
		 // create new file  
			   	   
     passwdFile = new File(newName);            
     
     // create write stream to the new file
     
     passwdFileWrite = new FileWriter(passwdFile);
     bWriteFile = new BufferedWriter(passwdFileWrite);
     
     
     // create line to be written in the file
      
     newLine = name + ":" + encPasswd + ":" + status + ":" + fullName + ":" + eMail;
     		         
	 
	   while(line != null) {    		   		  


//		      separatorPos = line.indexOf(':');
//		      userName = line.substring(0, separatorPos);


		      if (line.startsWith("#")) {
			
					    // it is comment line, copy it without modifications
					    
				      bWriteFile.write(line, 0, line.length());
					    bWriteFile.newLine();
					    bWriteFile.flush();			      
		      
		      } else {

			      // get name field from the line
			      
			      userName = getField(line, NAME);
		      
			      if (userName != null) {
			    
					    if (userName.equals(name)) {
					  
					       // user with name = 'name' already exists, the line will be overwritten
					     
					       existed = true;      						 
						     bWriteFile.write(newLine, 0, newLine.length());
						     bWriteFile.newLine();
						     bWriteFile.flush();
						 
					    } else {
					
					      // line will be copied otherwise
					
					      bWriteFile.write(line, 0, line.length());
					      bWriteFile.newLine();
					      bWriteFile.flush();
					  
					    }
					  }
					}
					
		      try {     
	         
			       // read another line
		       
		        line = bufferedFile.readLine();
       
		      } catch (Exception e) {
          
			       e.printStackTrace(Log.err);
       
		      }
                                                                            					
		}          
      
    // user with a name equal to 'name' did not exist in the file
    // new line to configuration file is added 
       
	  if (!existed) {	    																																			       																																																																				       
																																																																																																							       									  																	       		                                                       
			bWriteFile.write(newLine, 0, newLine.length());
			bWriteFile.newLine();
			bWriteFile.flush();																																																																																													      																																																																																																																	       																																											       																																																																																											       		   																	       																									       
		} 
		
		// close the files
		
		if (bufferedFile != null) {			
											
		  bufferedFile.close();	
		}
		
		bWriteFile.close();		
		oldPasswdFile.delete();  
								  																																																 
																																																																																					 							       																								                                                                         
  }
  
/*
public void removeEntry(String name) throws IOException
  
Removes user entry from the file. Works similar as addEntry();
*/  
  
   public void removeEntry(String name) throws IOException {
  
     String newName;   
     String oldName;
     String userName;
     int separatorPos;
     File oldFile;
     File oldPasswdFile;
     FileWriter passwdFileWrite;
	   BufferedWriter bWriteFile;
	   String line;
	   int namePos;
//	   boolean existed = false;
	   BufferedReader bufferedFile;
	   FileReader passwdFileRead;
	   File passwdFile;
	   
     if (name == null) {
     
       return;
     
     }
	 
	         
  //   encPasswd = getEncPasswd(decPasswd);          
  //   userTable.put(name, encPasswd);
        
	   newName = passwdFilename;
     oldName = newName + ".tmp";
//     oldFile = new File(oldName);
//     oldFile.delete();   
   
   
     oldPasswdFile = new File(oldName);
     passwdFile = new File(passwdFilename);
     passwdFile.renameTo(oldPasswdFile);     
//     oldPasswdFile = new File(oldName);
     
     try {
     
       passwdFileRead = new FileReader(oldPasswdFile);     
       bufferedFile = new BufferedReader(passwdFileRead);     
       line = bufferedFile.readLine();
       
     } catch(FileNotFoundException e) {
          
       line = null;
       bufferedFile = null;
     }
	   	   
     passwdFile = new File(newName);            
     passwdFileWrite = new FileWriter(passwdFile);
     bWriteFile = new BufferedWriter(passwdFileWrite);
      
    
	 
	   while(line != null) {    		   		  

//			    namePos = line.indexOf(name);
			    
//			    separatorPos = line.indexOf(':');
//		      userName = line.substring(0, separatorPos);

		      
				  if (line.startsWith("#")) {
		      
				      bWriteFile.write(line, 0, line.length());
					    bWriteFile.newLine();
					    bWriteFile.flush();			      
		      
		      } else {
    
						      		      
			      userName = getField(line, NAME);
		      
				      if (userName != null) {
			    
					      if (!userName.equals(name)) {
					   					 					
						      bWriteFile.write(line, 0, line.length());
						      bWriteFile.newLine();
						      bWriteFile.flush();
					  
					      }
					    }
					}
					
		      try {     
	         
		        line = bufferedFile.readLine();
       
		      } catch (Exception e) {
          
			       e.printStackTrace(Log.err);
       
		      }
                                                                            					
		}    
		 
		if (bufferedFile != null) {	
			        
		  bufferedFile.close();	
	  }
	  
		bWriteFile.close();			
		oldPasswdFile.delete();											  																																																 
																																																																																					 							       																								                                                                         
  }

/*

  private String getEncPasswd(String decPasswd) 
  
  Encrypts password


*/

  
  private String getEncPasswd(String decPasswd) {           
         
     String encPasswd;
     StringBuffer tmpPasswd = new StringBuffer();
     byte[] table;
     int i;
         
     md.update(decPasswd.getBytes());                     
     table = md.digest();
     
     for (i = 0; i < table.length - 1; i ++) {
     
       tmpPasswd.append(String.valueOf(table[i]) + ".");     
     
     }
     
     tmpPasswd.append(String.valueOf(table[i]));
               
     encPasswd = tmpPasswd.toString();

     return encPasswd;
  
  }

/*  
      
   public Hashtable getEntries(int column) 

   Returns Hashtable with valid entries of the form: (name, value in column 'column'). 
Name is the key in the hash table. Method returns null if configuration file
does not exit 
      
*/  
  
  public Hashtable getEntries(int column) {
  
    String line, user, data = null;
    FileReader passwdFileRead;      
    BufferedReader bufferedFile;
    Hashtable result = new Hashtable();
    File passwdFile;
  
    // open file for read
  
    try { 
       
       passwdFile = new File(passwdFilename);
       passwdFileRead = new FileReader(passwdFile);      	       	    
	    		      
    } catch(FileNotFoundException e) {
	    
	      passwdFile = null;
		    passwdFileRead = null;
	      return null;
	    
	  } catch(Exception e) {
	    
	      passwdFile = null;
	      passwdFileRead = null;
	      e.printStackTrace(Log.err);
	      return null;
	    
	  }
 
     // create stream for read
 
    bufferedFile = new BufferedReader(passwdFileRead);
 
    do {
      
      // read line
      
      try {
      
        line = bufferedFile.readLine();
	
      } catch(Exception e) {

        Log.out.println("PasswdManager: " + e, 2);
        line = null;
        System.exit(1);
      }
    
      // find information in the line

      if (line != null) {
      
	      if (!line.startsWith("#") && !line.startsWith(" ") && !line.startsWith("\n")){

		      
		      // if not comment line, retrieve name and data stored in column 'column'
		      
			    user = getField(line, NAME);			      			     			      
				  data = getField(line, column);				    				    
				    												    
			    if (user != null) {														    
																												    				    				    
				     if (data == null) {
				    
					     Log.out.println("PasswdManager: null data at position" + column, 2);
						   data = new String("");
				     }
				    			    
						// put data in hashtable
							    
				     result.put(user, data);
				     data = null;
				      
				  }
				  
			  }		         
			    //  System.out.println("PasswdManager: user added with parameters " + user + " " + password);
      }               	    

    } while(line != null);
        
    return result;

  }


/*

 private String get(String name, int column)
 
Retrieves data of the 'name' user that can be found on 'column' 
position in the line. If no data is specified, string of length 0 is returned (not null). 
If the user 'name' is not present in the file or entry
format is invalid, null is returned.

*/   
  private String get(String name, int column) {
  
    String line, user, field = null; 
    FileReader passwdFileRead;      
    BufferedReader bufferedFile;
    File passwdFile;

     // open file for read
   
    try { 
       passwdFile = new File(passwdFilename);
       passwdFileRead = new FileReader(passwdFile);      	       	    
	    		      
    } catch(FileNotFoundException e) {
	    
		    passwdFileRead = null;
	      return null;
	    
	  } catch(Exception e) {
	    
	      passwdFileRead = null;
	      e.printStackTrace(Log.err);
	      return null;
	    
	  }
 
    // open buffered reader for file read operation
 
    bufferedFile = new BufferedReader(passwdFileRead);
 
    do {
      
      // read one line
      
      try {
      
        line = bufferedFile.readLine();
	
      } catch(Exception e) {

        Log.out.println("PasswdManager: " + e, 2);
        line = null;
        System.exit(1);
      }

      if (line != null) {

        if (!line.startsWith("#") && !line.startsWith(" ") && !line.startsWith("\n")){
		     		      
				    // retrieve user name stored in this line  
				      
			      user = getField(line, NAME);
			      
			      // check if the user name is equal to 'name' method parameter
			      
			      if (user != null) 
			      			      
				      if (user.equals(name)) {
			      
					      // find value of data on 'column' position in the line
			      
					      field = getField(line, column);
				      
				      }				    				  
				}  
	    }

    } while(line != null && field == null);
    
    
    return field;        
  
  }
  
 /*
 private String getField(String line, int column)
 
 retrieves data from the line, stored on position 'column'
     
   
 */ 
  
  
  
  
  private String getField(String line, int column) {
  
    int start, end, pos1, pos2;
    String result = null;
    
    
    pos2 = getPosition(line, ENUMBER - 1);
    
    if (pos2 == -1) 
    
      return null;
    
    
    switch(column) {
    
      case NAME:
	      
	      start = 0;
	      pos1 = getPosition(line, column);
	      if (pos1 != -1) {
	      
		      result = line.substring(start, pos1);
	      
	      }
	      break;
      
      case ENUMBER:
      
	      pos1 = getPosition(line, column - 1);
	      end = line.length();
	      
	      if (pos1 != -1) {
	      
		      result = line.substring(pos1 + 1, end);
	      
	      }
	      break;	      
      
    
      default:
      
	      pos1 = getPosition(line, column - 1);
	      pos2 = getPosition(line, column);
	      
	      if (pos1 != -1 && pos2 != -1) {
	      
		      result = line.substring(pos1 + 1, pos2);
	      
	      }
	                  
    }
    
	
	  return result;      
      
  }
 
  /*
  
  private int getPosition(String line, int occurence)
  
  returns position of i-th occurence of ':' character in the line
  
  
  
  */
   
     
  private int getPosition(String line, int occurence) {
  
    int position = 0;
    int tmp = 0;
  
    while (occurence > 0 && position != -1) {
    
      position = line.indexOf(":", tmp + 1);
      tmp = position;        
      occurence --;
    }
    
    return position;
  
  }
        
}
