// $Id: CentralServer.java,v 1.7 1999/04/21 21:45:29 beca Exp $
/*
File: CentralServer.java
Component: Plugin
Date		Author		Changes
Mar 07 97	Lukasz Beca	Created
*/

package webwisdom.tango.structures;

import webwisdom.tango.structures.ControlApplication;
import webwisdom.tango.structures.LocalApplications;
import webwisdom.tango.threads.DemonThreadIn;
import webwisdom.tango.threads.ThreadOut;
import webwisdom.tango.messages.*;
import java.net.Socket;
import java.io.*;
import webwisdom.tango.consts.Const;
import webwisdom.tango.test.Log;
//import netscape.security.PrivilegeManager;


public class CentralServer
{
    private static final String CL="CentralServer";


    private ControlApplication ca;
    private LocalApplications apps;
    private DemonThreadIn in;
    private ThreadOut out;
    private String serverhost;
    private int serverport;


    public CentralServer()
    {
      ca = null;
      apps = null;
      in = null;
      out = null;
      serverhost = null;
      serverport = 0;
    }


    //interface for LocalBase
    //public void initAssociations(ControlApplication capar,LocalApplications lapar);
 

    public void initAssociations(ControlApplication capar,
				 LocalApplications lapar)
    {
      ca = capar;
      apps = lapar;
//      initSocket();
    }
   

    //interface for ControlApplication
    //public void receiveFromCA(Message m);
    //public void logout();
    //public String getServerHost();


    public void sendCAToCS(Message m)
    {
      out.putMessage(m);
    }
  

    public String getServerHost()
    {
      return serverhost;
    }


    public void logout()
    {
      in.logout();
    }
  
    public void openConnection(String hostname, int port) throws IOException {
    
        serverhost = hostname;
       	serverport = port;
        Log.out.println(CL + ": openConnection " + hostname + " " + port, 2);
        initSocket(hostname, port);                 
    }
    
    
    public void closeConnection() throws IOException {
        
//	      serverhost = null;
//	      serverport = 0;
//      reset();	  
        Log.out.println(CL + ": closeConnection", 2);    
	      logout();
    }
     

    //interface for LocalApplications
    //public void receiveFromApp(Message m);
    //public void started(int AID);


    public void sendLAToCS(Message m)
    {
      out.putMessage(m);
    }


    public void started(int AID, int SID)
    {
      in.started(AID, SID);
    }
       

    //interface for DemonProtocol
    //public void sendCSToCAB(Message m);
    //public void sendCSToApp(Message m);


    public void sendCSToCAB(Message m)
    {
      ca.sendCSToCAB(m);                
    }


    public void sendCSToApp(EventMessage m)
    {
      apps.sendCSToApp(m);
    }


    //interface for DemonThreadIn
    //public void connectionBrk();


    public void connectionBrk()
    {
      ca.errorServer(Const.CONNBROKEN);
     //end();
      reset();
    }


    //internal methods
    //private void initSocket();
    //public void end();


    public void initSocket(String hostname, int port) throws IOException  
    {
      Socket s;
      if (serverhost.compareTo("*") == 0)
      {
	hostname = null;
	port = 0;     
	ca.errorServer(Const.FILEWRONGFORMAT);
	return;
      }
      if(port <= 0)
      {
	hostname = null;
	port = 0;      
	ca.errorServer(Const.FILEWRONGFORMAT);
	return;
      }
      if(Const.inNetscape())
	{
	  netscape.security.PrivilegeManager.enablePrivilege("30Capabilities");
	}
//      try
//      {
	s=new Socket(hostname,port);
//      }
//      catch(Exception e)
//      {
//	s = null;
//	System.out.println(CL+".initSocket(): unable to contact server: "+e);
//	e.printStackTrace();
//	ca.errorServer(Const.UNABLETOCONNECT);
//	return;
//     }
      out = new ThreadOut(s);   
      in = new DemonThreadIn(out, s, this);
    }// initSocket _________________________________________________________
 

    public void end()
    {
      //   System.out.println("CentralServer: releasing references"); 
      ca = null;
      apps = null;
      if (in != null)
      {
	in.stop();
	in = null;
      }
      if (out != null)
      {
	out.stop();   
	out = null;
      }
      serverhost = null;
      serverport = 0;
    }
    
    
   private void reset() {
  
      if (in != null) {
              in.stop();
              in = null;
      }
      
      if (out != null) {
              out.stop();   
              out = null;
      }
      
      serverhost = null;
      serverport = 0; 
           
  
  }
 


}// CentralServer //////////////////////////////////////////////////////////

