// $Id: LocalApplications.java,v 1.7 1999/04/07 20:32:30 beca Exp $
/*
File: LocalApplications.java
Component: Plugin
Date		Author		Changes
Sept 06 96	Lukasz Beca	Created

Tested:

Notes:

*/

package webwisdom.tango.structures;

import java.util.Hashtable;
import java.util.Enumeration;
import webwisdom.tango.structures.Handle;
import webwisdom.tango.structures.ControlApplication;
import webwisdom.tango.structures.CentralServer;
import webwisdom.tango.messages.EventMessage;
import webwisdom.tango.messages.AppEventMessage;
import webwisdom.tango.messages.GroupEventMessage;
import webwisdom.tango.consts.Const;
import webwisdom.tango.test.Log;


/**Stores information about applications and applets connected to local demon*/
public class LocalApplications implements Runnable
{ 
    private static final String CL="LocalApplications";
 

    private ControlApplication ca;
    private CentralServer cs;
    private Hashtable HandleTable;

    
    public LocalApplications()
    {
      HandleTable = new Hashtable();
    }


    //interface for LocalBase
    //initAssociations(ControlApplication ca, CentralServer cs);


    public void initAssociations(ControlApplication capar,CentralServer cspar)
    {
      ca = capar;
      cs = cspar;
    }
  

    //interface for registrar
    //public synchronized void addApplication(Handle h,int a,int s);


    /**
     *@param h handle to application or applet
     *@param a AID*/
    public synchronized void addApplication(Handle h,int a,int s)
    {
      //System.out.println(CL+".addApplication("+a+")");
      Integer b;
      boolean added;   
      if(a==0)
      {
	Log.err.println(CL+".addApplication(): invalid aid!", 2);
	return;
      }
    
      b=new Integer(a);
      added=HandleTable.containsKey(b);
      if(!added)
      {
	try
	{
	  HandleTable.put(b,h);
	}
	catch(Exception e)
	{
	  Log.err.println(CL+".addApplication(): invalid handle!: "+e, 2);		  
	  e.printStackTrace(Log.err);
	}
	cs.started(a,s);
      }
      else
	Log.err.println(CL+".addApplication(): application already added!", 2);
    }// addApplication _____________________________________________________


    //interface for application Handles
    //public void terminated(int AID);
    //public void sendAppToOther(int AID, AppEventMessage m);
    //public void sendAppToOther(int AID, int userlist[], AppEventMessage m);


    public void terminated(int AID)
    {
      Integer AIDtmp = new Integer(AID);
      ca.removeApplication(AID);
      HandleTable.remove(AIDtmp);
    }


    public void sendAppToOther(int AID,AppEventMessage m)
    {
      int type = m.getType();
      byte data[] = m.getData();
      EventMessage a;
      switch(type)
      {
	case Const.DATA:
	  a = new EventMessage(AID, data);
	  cs.sendLAToCS(a);
	  break;
	case Const.CONTROL:
	  ca.sendLAToCAB(AID, m);
	  break;
	case Const.PING:
	  //System.out.println(CL+".sendAppToOther(PING)");
	  break;
	default:
	  Log.err.println(CL+".sendAppToOther(): unknown message type: "+type+"!", 2);
      }
    }
  
  
    public void sendAppToOther(int AID,int userlist[],AppEventMessage m)
    {
      int type = m.getType();
      byte data[] = m.getData();
      GroupEventMessage a;
      if (type == Const.DATA)
      {
	a = new GroupEventMessage(AID, userlist, data);
	cs.sendLAToCS(a);
      }
    }


    //interface for ControlApplication
    //public synchronized boolean appInSession(int AIDpar);
    //public synchronized void finish(int AIDpar);
    //public synchronized void finishAll();
    //public void setFocus(int AIDpar);
    //public void sendCAtoApp(int AID, AppEventMessage m);
    //public void end();
    

    public synchronized boolean appInSession(int AIDpar)
    {
      Integer AID = new Integer(AIDpar);
      return HandleTable.containsKey(AID);
    }


    public synchronized void finish(int AIDpar)
    {
      Integer AID=new Integer(AIDpar);    
      Handle h=(Handle)HandleTable.remove(AID);
      if(h!=null)
	h.finish();    
      else
     	Log.err.println(CL+".finish("+AIDpar+"): no such application!", 2);
    }


    public synchronized void finishAll()
    {
/*      Handle h;
      Integer key;
      int i, size;
      Enumeration e;
      e = HandleTable.keys();
      
      Log.out.println(CL+".finishAll()", 2);
      while(e.hasMoreElements())
      {
	key = (Integer) e.nextElement();      
	h = (Handle) HandleTable.remove(key);
	h.finish();
      }  */
      
      Thread appKiller = new Thread(this, "LocalApplications:" + hashCode());
      appKiller.start();
            
    }
    
    
    public void run() {
    
      Handle h;
      Integer key;
      int i, size;
      Enumeration e;
      e = HandleTable.keys();
      
      Log.out.println(CL+".run()", 2);
      while(e.hasMoreElements())
      {
	      key = (Integer) e.nextElement();      
	      h = (Handle) HandleTable.remove(key);
	      h.finish();
      }            
    
    }


    public void setFocus(int AIDpar)
    {
      Integer AID = new Integer(AIDpar);   
      Handle h=(Handle)HandleTable.get(AID);
      if(h!=null)
	h.setFocus();
      else
     	Log.err.println(CL+".setFocus("+AIDpar+"): no such application!", 2);
    }

  
    public void sendCAToApp(int AID,AppEventMessage m)
    {
      Integer AIDtmp= new Integer(AID);
      Handle h= (Handle) HandleTable.get(AIDtmp);
      if (h != null)
	h.sendLAToApp(m);
      else
      	Log.err.println(CL+"sendCAToApp("+AID+"): no such application!", 2);
    }


    public void end()
    {
      ca = null;
      cs = null;
//      HandleTable = null; check impact
    }


    //interface for CentralServer
    //public void sendCStoApp(EventMessage m)


    public void sendCSToApp(EventMessage m)
    {
      int AID=m.getAID();
      byte data[]=m.getData();
      AppEventMessage appm;
      Integer AIDtmp= new Integer(AID);
      Handle h= (Handle) HandleTable.get(AIDtmp);
      if(h!=null)
      {
	appm = new AppEventMessage(Const.DATA, data);
	h.sendLAToApp(appm);
      }     
      else
      	Log.err.println(CL+"sendCAToApp(): no such application: "+AID+"!", 2);
    }

  
    //diagnostic methods
    //public void list();


    public void list()
    {
      Enumeration e= HandleTable.keys();
      Log.out.println(CL+".list():", 2);
      while(e.hasMoreElements())
      {
      	Log.out.println(" AID="+e.nextElement(), 2);
      }
    }


}// LocalApplications //////////////////////////////////////////////////////



