// $Id: ScriptHandle.java,v 1.8 1999/03/30 18:48:30 beca Exp $
/*
File: ScriptHandle.java
Component: Plugin
Date		Author		Changes		
Jan 21 97	Lukasz Beca	Created
Apr 21 98	Tomasz Major	modified run() and putMessage(String),
                                added putBytesMessage(), putStringMessage()
*/

package webwisdom.tango.structures;

import webwisdom.tango.structures.Handle;
import webwisdom.tango.interfaces.AppletBase;
import webwisdom.tango.messages.AppEventMessage;
import webwisdom.tango.messages.EmptyMessage;
import webwisdom.tango.messages.Message;
import webwisdom.tango.structures.LocalApplications;
import webwisdom.tango.messages.EventMessage;
import webwisdom.tango.consts.Const;
import webwisdom.tango.consts.CAConsts;
import netscape.javascript.JSObject;
import webwisdom.tango.messages.AppEventMessage;
import webwisdom.tango.messages.CAppRequestEventMessage;
import webwisdom.tango.messages.CAppAnswerEventMessage;
import webwisdom.tango.consts.CAConsts;
import webwisdom.tango.test.Log;
import java.io.*;

public class ScriptHandle implements Handle, Runnable {
    private static final String CL="ScriptHandle";


    private JSObject script;
    private int AID;
    private LocalApplications lapps;
    
    private MessageQueue q;
    private boolean exit; 
    private Thread th;
    
    final static int DATA = 1;
    final static int CONTROL = 0;

    static final byte FRAMESMES=(byte)203;
    

    public ScriptHandle(JSObject s, int AIDpar, LocalApplications localpar)
    {
      script = s;
      AID = AIDpar;
      lapps = localpar;
      q = new MessageQueue();
      th = new Thread(this, "ScriptHandle:" + hashCode());
      exit = false;
      th.start();
    }


    public void run()
    {
      Object args[] = new Object[2];
         
      while(!exit)
      {
	//System.out.println(CL+": Thread number " + thisno + " sending");
	Message m=q.getMessage();	
	if(!exit)
	{
	  int type=m.getType();
	  if(type==Const.CONTROL)
	  {
	    AppEventMessage aem=(AppEventMessage)m;
	    byte b[]=aem.getData();
	    int ctype=(int)b[0];
	    if(ctype==CAConsts.ISMASTER)
	    {
	      if(b[1]==CAConsts.YES)
	      {
		script.call("Tango_setMaster", null); 			
	      }
	      else
	      {
		script.call("Tango_setSlave", null);
	      }
	    }
	    else if(ctype==FRAMESMES)
	    {
	      ByteArrayInputStream bais=new ByteArrayInputStream(b);
	      DataInputStream dis=new DataInputStream(bais);
	      try
	      {
		dis.readByte();//ctype
		int len=dis.readInt();
		for(int i=0;i<len;i++)
		{
		  String appl=dis.readUTF();
		  String fram=dis.readUTF();
		  args[0]=appl;
		  args[1]=fram;
		  script.call("Tango_respondWindows",args);
		}
	      }
	      catch(IOException e)
	      {
		Log.err.println(CL+": error processing 203 message!: "+e, 2);
		e.printStackTrace(Log.err);
	      }
	      finally
	      {
		try
		{
		  dis.close();
		}
		catch(IOException e)
		{
		}
	      }
	    }
	  //System.out.println("Got:"+ht);
	  }
// 	  else {
// 	    am = (AppEventMessage) m;
// 	    data = am.getData();
// 	    args[0] = bytesToString(data);
// 	    script.call("Tango_receive", args); 
// 	  }
	  else
	  {
	    AppEventMessage am=(AppEventMessage)m;
	    byte[] data=am.getData();
	    int chId=(int)data[0];
	    int chDataLen=data.length-1;
	    if(chId==-128)/*Strings, no channels*/
	    {
	      String chData=new String(data,0,1,chDataLen);
	      args[0]=chData;
	      script.call("Tango_receive",args);
	    }
	    else if(chId>=0)/*bytes*/
	    {
	      String chName=String.valueOf(chId);
	      byte[] chData=new byte[chDataLen];
	      for(int i=0;i<chDataLen;i++)
		chData[i]=data[1+i];
	      args[0]=chName;args[1]=chData;
	      script.call("Tango_receiveBytes",args);
	    }
	    else/*Strings*/
	    {
	      chId=-chId;
	      String chName=String.valueOf(chId);
	      String chData=new String(data,0,1,chDataLen);
	      args[0]=chName;args[1]=chData;
	      script.call("Tango_receiveString",args);
	    }
	  }
	}
      }
    }// run ________________________________________________________________


    public void askWindows()
    {
      byte[] b=new byte[1];
      b[0]=FRAMESMES;
      AppEventMessage aem=new AppEventMessage(Const.CONTROL,b);  
      lapps.sendAppToOther(AID,aem);
    }// askWindows _________________________________________________________


    /*The following 3 functions concatenate channel id and channel data
      into one message:
      first byte - channel number, remaining bytes - data.
      As channels are represented by one byte
      the channel number must be within [-128,127].
      Api with Strings uses channels [-128,-1], with bytes - [0,127].
      Default channel for Strings api without channels is -128.
      User uses channels [0,127] only,
      the rest is used to save information which api is in use.*/


    /**Sends byte[] through the given channel.
     *@param chName channel name (must represent a number 0-127) eg "13"
     *@param chData channel data*/
    public void putBytesMessage(String chName,byte[] chData)
    {
      int chId=Integer.valueOf(chName).intValue();
      int chDataLen=chData.length;
      byte[] data=new byte[1+chDataLen];
      data[0]=(byte)chId;
      for(int i=0;i<chDataLen;i++)
	data[1+i]=chData[i];
      lapps.sendAppToOther(AID,new AppEventMessage(Const.DATA,data));
    }// putBytesMessage ____________________________________________________


    /**Sends String through the given channel.
     *@param chName channel name (must represent a number 1-127) eg "14"
     *@param chData channel data*/
    public void putStringMessage(String chName,String chData)
    {
      int chId=-Integer.valueOf(chName).intValue();
      int chDataLen=chData.length();
      byte[] data=new byte[1+chDataLen];
      data[0]=(byte)chId;
      chData.getBytes(0,chDataLen,data,1);
      lapps.sendAppToOther(AID,new AppEventMessage(Const.DATA,data));
    }// putBytesMessage ____________________________________________________


    /**Sends String by default channel.
     *@param chData data
     *@see #putStringMessage*/
    public void putMessage(String chData)
    {
      int chDataLen=chData.length();
      byte[] data=new byte[1+chDataLen];
      data[0]=(byte)-128;
      chData.getBytes(0,chDataLen,data,1);
      //l.sendServer(new EventMessage(AID,data));
      lapps.sendAppToOther(AID,new AppEventMessage(Const.DATA,data));
    }// putMessage _________________________________________________________


//   public void putMessage(String info) {

//     byte data[] = stringToBytes(info); 
//     AppEventMessage a;
   
//     a = new AppEventMessage(Const.DATA, data);
//     lapps.sendAppToOther(AID, a);
 

//   }


  public void askMaster() {

     byte[] tab = new byte[1];
     tab[0] = 1; //question if applet is a master or not
     CAppRequestEventMessage mes = new CAppRequestEventMessage(tab);  
	
     lapps.sendAppToOther(AID, mes);
    
  }


  public void sendLAToApp(Message m) {
    

       q.putMessage(m);

  }

  
  public void finish() {

       script.call("Tango_finish", null);
//     script.eval("close();");

  }

  public void remove() {
 //    System.out.println("ScriptHandle: removing script " + AID);
     exit = true;
     q.unblock();
     lapps.terminated(AID);
  }

  public void setFocus() {

     script.eval("focus();");
  }
  

//   private String bytesToString(byte data[]) {

//     String result;

//     result = new String(data, 0);

//     return result;

//   }


//   private byte[] stringToBytes(String data) {

//     byte result[];
//     int len;

//     len = data.length();
//     result = new byte[len];
//     data.getBytes(0, len, result, 0);

//     return result;

//   }


    public String toString()
    {
      return CL+"[aid="+AID+",scr="+script+"]";
    }// toString ___________________________________________________________


}




