// $Id: AppThreadOut.java,v 1.9 1999/03/30 18:48:36 beca Exp $
/*
File: AppThreadOut.java
Component: Plugin
Date		Author		Changes
Sept 06 96	Lukasz Beca	Created
*/


package webwisdom.tango.threads;


import java.lang.*;
import java.net.*;
import java.io.*;
import webwisdom.tango.messages.AppNet;
import webwisdom.tango.messages.Message;
import webwisdom.tango.structures.MessageQueue;
import webwisdom.tango.consts.Const;
import webwisdom.tango.messages.AppEventMessage;
import webwisdom.tango.test.Log;



/**Demon thread.*/
public class AppThreadOut implements Runnable
{
    private static final String CL="AppThreadOut";


    private MessageQueue q;
    private AppNet n;
    private Thread th;
    private boolean exit;
    

    static int number=1;    //for testing only
    int thisno;  //for testing only

    private Thread pinger;


    public AppThreadOut(Socket s)
    {
      super();    

      thisno=number;
      number++;

      n=new AppNet(s);
      q=new MessageQueue();
      th=new Thread(this, "AppThreadOut:" + hashCode());
      exit=false;
      th.start();

      pinger=new Thread(new Pinger(n,600000), "Pinger:" + hashCode());
      pinger.start();
    }// AppThreadOut _______________________________________________________


    public void run()
    {
      Log.out.println(CL+".run(): "+thisno, 2);
      while(!exit)
      {
	Message m=q.getMessage();
	try
	{
	  n.send(m);
          //System.out.println(CL+": "+thisno+" sending "+m);
	}
	catch(IOException e)
	{
	  Log.err.println(CL+": error sending message: "+e, 2);
	  e.printStackTrace(Log.err);
	}
      }
    }// run ________________________________________________________________


    public synchronized void putMessage(Message m)
    {
      q.putMessage(m);
    }// putMessage _________________________________________________________

 
    public void stop()
    {
      //System.out.println(CL+".stop()");
      exit = true;
      q.unblock();
      pinger.stop();
    }// stop _______________________________________________________________


}// AppThreadOut ///////////////////////////////////////////////////////////




class Pinger implements Runnable
{
    private static final String CL="Pinger";


    private Thread _th;
    private int _interval;
    private AppNet n;


    public Pinger(AppNet npar,int time)
    {
      n=npar;
      _interval=time;
    }// Pinger ____________________________________________________________
    

    public void run()
    {
      byte[] data=new byte[1];
      data[0]=Const.UNUSED;
      AppEventMessage pm=new AppEventMessage(Const.PING,data);

      //System.out.println(CL+".run()");
      while(true)
      {
	try
	{
	  Thread.sleep(_interval);
	}
	catch(InterruptedException e)
	{
	  Log.err.println(CL+": interruption: "+e, 2);
	  e.printStackTrace(Log.err);
	}
	try
	{
	  n.send(pm);
	  //System.out.println(CL+": sent");
	}
	catch(IOException e)
	{
	  Log.err.println(CL+": error sending message: "+e, 2);
	  e.printStackTrace(Log.err);
	}
      }
    }// run ________________________________________________________________


}// Pinger /////////////////////////////////////////////////////////////////



