// $Id: Timeout.java,v 1.6 1998/12/19 22:18:41 beca Exp $
/*
File: Timeout.java
Component: Server
Date		Author		Changes
Jan 23 98	Lukasz Beca	Created
*/


package webwisdom.tango.threads;


import java.util.*;
import webwisdom.tango.threads.*;
import webwisdom.tango.protocols.*;
import webwisdom.tango.messages.*;
import webwisdom.tango.consts.*;
import webwisdom.tango.test.Log;


class StateEntry
{
    private static final String CL="StateEntry";


    private ServerProtocol _in;
    private byte _state;


    public StateEntry(ServerProtocol in)
    {
      _state = Const.OK;
      _in = in;
    }// StateEntry _________________________________________________________

	
    public int getState()
    {
      return _state;
    }// getState ___________________________________________________________

	
    public void setState(byte state)
    {
      _state = state;
    }// setState ___________________________________________________________

    public void deleteUser()
    {
      _in.logout(Const.TIMEOUT);
      _in.stop();
    }// deleteUser _________________________________________________________


}//  StateEntry ////////////////////////////////////////////////////////////




public class Timeout implements Runnable
{
    private static final String CL="Timeout";


    private Hashtable _pingtable;
    private Thread _th;
    private ComThread _ct;
    private int _interval;
	

    public Timeout(int time, ComThread ct)
    {
      _pingtable=new Hashtable();
      _ct=ct;
      _interval=time;
      _th=new Thread(this);
      _th.start();
    }// Timeout ____________________________________________________________
    

    public void userResponded(int UID)
    {
      StateEntry se=(StateEntry)_pingtable.get(new Integer(UID));
      se.setState(Const.OK);
    }// userResponded ______________________________________________________
	

    public void userRemoved(int UID)
    {
      _pingtable.remove(new Integer(UID));
    }// userRemoved ________________________________________________________
	
    
    public void newUser(int UID, ServerProtocol in)
    {
      StateEntry se=new StateEntry(in);
      //System.out.println(CL+".newUser()");
      _pingtable.put(new Integer(UID), se);
    }// newUser ____________________________________________________________
    

    public void run()
    {
      PingMessage pm=new PingMessage();
      Log.out.println(CL+".run()", 2);
      while(true)
      {
	try
	{
	  Thread.sleep(_interval);
	}
	catch(InterruptedException e)
	{
	 Log.err.println(CL+": interruption: "+e, 2);
	  e.printStackTrace(Log.err);
	}
	Enumeration elems=_pingtable.elements();
	Enumeration keys=_pingtable.keys();
	while(elems.hasMoreElements())
	{
	  StateEntry se=(StateEntry)elems.nextElement();
	  int UID=((Integer)keys.nextElement()).intValue();
	  if(se.getState()==Const.OK)
	  {
	    _ct.sendTo(UID,pm);
	    se.setState(Const.TESTING);
	  }
	  else
	  {
	   Log.out.println(CL+": user "+UID+" crashed", 2);
	    se.deleteUser();
	  }						
	}								
      }
    }// run ________________________________________________________________


}// Timeout ////////////////////////////////////////////////////////////////

